"""
Flyer - Python Framework for Flash Lite developers.
Copyright (C) 2007  Felipe Andrade

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""

import appuifw
import e32

class Flyer:
    def __init__(self):
        import socket
        serv = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        serv.bind(('127.0.0.1', 9100))
        serv.listen(999)
        self.serv = serv
        self.running = 0
        self.commands = {}
        self.state = None

    # start()
    # Starts the server and commands   
    def start(self):        
        self.commands["takePhoto"] = self.takePhoto
        self.commands["takeScreenshot"] = self.takeScreenshot
        self.commands["sendFileBT"] = self.sendFileBT
        self.commands["getContacts"] = self.getContacts
        self.commands["playSound"] = self.playSound
        self.commands["recordSound"] = self.recordSound
        self.commands["stopSound"] = self.stopSound
        self.commands["sayText"] = self.sayText
        
        self.running = 1
        while (self.running == 1):
            print u"Flyer 1.0.35 - Waiting for FL connection"
            (client, addr) = self.serv.accept()
            self.client = client
            self.addr = addr
            print u"Connection opened with ", self.addr
            self.client.recv(1024, cb=self.handle)
            
    # broadcastMessage()
    # Broadcast the message to connected clients.
    def broadcastMessage(self, text):
        text += '\0'
        self.client.send(text)
        
    def takePhoto(self, filePath):
        from flyergraphics import FlyerGraphics
        fGraphics = FlyerGraphics()
        fGraphics.takePhoto(filePath, callback=self.broadcastMessage)
        self.state = fGraphics
        
    def takeScreenshot(self, filePath):
        from flyergraphics import FlyerGraphics
        fGraphics = FlyerGraphics()
        fGraphics.takeScreenshot(filePath, callback=self.broadcastMessage)
        self.state = fGraphics
        
    # sendFileBT()
    # Send files over a bluetooth connection
    def sendFileBT(self, filePath):
        from flyernetwork import FlyerNetwork
        fNetwork = FlyerNetwork()
        fNetwork.sendFileBT(filePath, callback=self.broadcastMessage)
        self.state = fNetwork

    # play()
    # Play sound
    def playSound(self, filePath):
        if self.state is not None:
            self.state.play(callbackSound=self.broadcastMessage)
        else:
            from flyersound import FlyerSound
            if filePath is not None:
                fSound = FlyerSound(filePath)
            else:
                fSound = FlyerSound()
            fSound.play(callbackSound=self.broadcastMessage)
            self.state = fSound
        
    # record()
    # Record sound
    def recordSound(self, filePath):
        if self.state is not None:
            self.state.record(callbackSound=self.broadcastMessage)
        else:
            from flyersound import FlyerSound
            if filePath is not None:
                fSound = FlyerSound(filePath)
            else:
                fSound = FlyerSound()
            fSound.record(callbackSound=self.broadcastMessage)
            self.state = fSound
    
    # stop()
    # Stop sound
    def stopSound(self, info=None):
        if self.state is not None:
            self.state.stop(callbackSound=self.broadcastMessage)

    # say()
    # Convert text to audio
    def sayText(self, sentence):
        from flyersound import FlyerSound
        fSound = FlyerSound()
        fSound.say(sentence, callbackSound=self.broadcastMessage)
        self.state = fSound

    def getContacts(self, db=None):
        from flyerdatabase import FlyerDatabase
        fDB = FlyerDatabase()
        fDB.getContacts(callback=self.broadcastMessage)
        self.state = fDB
    
    def handle(self, msg=None):
        msg = msg.replace("\0", "")
        data = msg.split("|")

        if msg is not None:
            cmd = data[0]      # command name
            filePath = data[1] # filePath 'E:\\dir\\filename.png'
        
        try:
            self.commands[cmd](filePath)
        except KeyError:
            print "Invalid Command"
        self.client.recv(1024, cb=self.handle)
            
    def quit(self):
        self.client.close()
        self.running = 0
        self.serv.close()
        appuifw.app.set_exit()

flyer = Flyer()
flyer.start()

appuifw.app.title = u"Flyer"
appuifw.app.exit_key_handler = flyer.quit
