"""
Flyer - Python Framework for Flash Lite developers.
Copyright (C) 2007  Felipe Andrade

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""

from appuifw import *
from audio import *
import audio

class FlyerSound:
    def __init__(self, filePath=None):
        if(filePath is None):
            filePath = "E:\\Flyer\\recordedSound.wav"
        self.sound = Sound.open(filePath)
        self.sound.set_volume(self.sound.max_volume())
        self.filePath = filePath
                    
    def play(self, callbackSound=None):
        if(self.sound.state() == ERecording):
            if callbackSound is not None:
                callbackSound(u"Stop sound to continue.")
            else:
                print "Stop sound to continue."
        else:
            if(self.sound.state() == EPlaying):
                if callbackSound is not None:
                    callbackSound(u"Stop sound playback.")
                else:
                    print "Stop sound playback."
            else:
                try:
                    if callbackSound is not None:
                        self.sound.play()
                        callbackSound(u"Playing sound.")
                    else:
                        self.sound.play()
                except Exception, error:
                    if callbackSound is not None:
                        callbackSound(u"An error occured while starting to play an audio file.")
                    else:
                        print "An error occured while starting to play an audio file."

    def record(self, callbackSound=None):
        if(self.sound.state() == ERecording):
            if callbackSound is not None:
                callbackSound(u"Stop sound to continue.")
            else:
                print "Stop sound to continue."
        else:
            if(self.sound.state() == EPlaying):
                if callbackSound is not None:
                    callbackSound(u"Stop sound playback.")
                else:
                    print "Stop sound playback."
            else:
                self.sound.record()
                callbackSound(u"Recording.")
                
    def say(self, sentence=None, callbackSound=None):
        if sentence is None:
            sentence = query(u'Please provide the text', 'text')
        try:            
            audio.say(sentence)
            if callbackSound is not None:
                callbackSound(u'Audio finished.')
        except Exception, error:
            if callbackSound is not None:
                callbackSound(unicode(error[1]))
            else:
                print unicode(error[1])
        
        
    def stop(self, callbackSound=None):
        if(self.sound.state() == ERecording):
            self.sound.stop()
            self.sound.close()
            self.sound = Sound.open(self.filePath)
            if callbackSound is not None:
                callbackSound(u"Audio stopped.")
            else:
                print "Audio stopped."
        else:
            self.sound.stop()
            if callbackSound is not None:
                callbackSound(u"Audio stopped.")
            else:
                print "Audio stopped."

    def quit(self):
        self.sound.close()
        appuifw.app.set_exit()

def test():
    import e32    
    
    script_lock = e32.Ao_lock()

    appuifw.app.title = u"FlyerSound"
    flyerSound = FlyerSound("E:\\Flyer\\recordedSound.wav")
    appuifw.app.menu = [(u"Play", flyerSound.play),(u"Record", flyerSound.record),(u"Say", flyerSound.say),(u"Stop", flyerSound.stop)]
    appuifw.app.exit_key_handler = flyerSound.quit

    script_lock.wait()

if __name__ == '__main__':
    test()
