'''
Roll a Ball - Useless but Fun Sensor API Application
Copyright (c) 2008 Jouni Miettunen

http://jouni.miettunen.googlepages.com/
Email: jouni dot miettunen at iki dot fi

2.0  2008-08-03 Full screen from beyond edge to edge
                More than bubbles: circle, plus, x, bar, triangle
                Hex layout
                Caleidoscope
                Bubble attraction with more bubbles
1.5  2008-07-23 Sensor color
1.4  2008-07-09 Quadruple Visions
1.3  2008-07-05 Experimental Visions
1.2  2008-06-18 Stabilize, optimize and beautify
1.1  2008-06-16 Power management tweaks (thanx aaaaapo)
1.0  2008-06-15 Something I've wanted to do for a long time

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

VERSION = u'2.0'

import e32
import appuifw
import graphics
import key_codes
import math
import random

# For HELP menu item
import keycapture
import TopWindow

# SENSOR: if exception, it's not available in this device
SENSOR_ACC = False
try:
    import sensor
    from sensor import orientation
    SENSOR = True                # Sensor interface available
except ImportError:
    SENSOR = False               # Sensor interface not available

# Global definitions for directions
MOVE_NONE = 0
MOVE_UP = 1
MOVE_DOWN = 2
MOVE_LEFT = 3
MOVE_RIGHT = 4

# Sensor related defintions
S_MAX = 100
S_MAX2 = 2 * S_MAX
S_MIN = -S_MAX

# Colors for different directions
RGB_MIN = 0
RGB_MAX = 255
RGB_MAX2 = RGB_MAX / 2
RGB_BLACK = (0, 0, 0)
RGB_WHITE = (255, 255, 255)

# Global variables
canvas = None
img = None
g_quad = 0
g_cale = 0
g_hex = 1
g_shape = 0
g_color = False
c_color = (0, 0, 0)
tm1 = tm4 = None
tm4_m1 = tm4_m2 = tm4_m3 = tm4_m4 = None
g_moving = False
orig_size = 26
my_size = orig_size
my_off = 0
my_bg = 2
top_x = 0
top_y = 0
g_atra = 0

# Board size and drawing related
dim_x = dim_y = off_x = off_y = 0
(rgb_red, rgb_green, rgb_blue) = RGB_WHITE
dir_red = dir_green = dir_blue = 1
sensor_x = sensor_y = 1
s_count = 0
s_data1 = s_data2 = 0

# Manage screensaver on/off
my_timer = e32.Ao_timer()
g_screensaver_on = True
sensor_acc = None

# Help menu item control
capturer = None

def new_color():
    ''' Change color, nothing else '''
    global rgb_red, rgb_green, rgb_blue
    (rgb_red, rgb_green, rgb_blue) = get_color()
    draw_bubbles(top_x, top_y)

def init_variables():
    ''' Speed up graphics with pre-calculated values '''
    global dim_x, dim_y, off_x, off_y, top_x, top_y
    global rgb_red, rgb_green, rgb_blue

    (rgb_red, rgb_green, rgb_blue) = get_color()
    x, y = canvas.size
    dim_x = x / my_size
    dim_y = y / my_size
    off_x = (x - dim_x * my_size) / 2
    off_y = (y - dim_y * my_size) / 2
    top_x = dim_x / 2
    top_y = dim_y / 2

    global sensor_x, sensor_y
    sensor_x = (S_MAX2 / dim_x)
    sensor_y = (S_MAX2 / dim_y)

def handle_sensor_raw(a_data):
    ''' My own handler for raw sensor events '''

    # Hox: Avoid unnecessary calculations
    if g_moving:
        return

# No need for slowing, heavy calculations elsewhere
    # Try to stabilize movement
#     global s_count, s_data1, s_data2
#     if s_count < 2:
#         s_count += 1
#         s_data1 += a_data['data_1']
#         s_data2 += a_data['data_2']
#         return
#
#     g_moving = True
#
#     data_1 = s_data1 / s_count
#     data_2 = s_data2 / s_count
#     s_data1 = s_data2 = s_count = 0
    # End of stabilize

    data_1 = a_data['data_1']
    data_2 = a_data['data_2']

    if g_color:
        b = sensor_to_color(a_data['data_3'])
        r = sensor_to_color(data_1)
        g = sensor_to_color(data_2)
        global c_color
        c_color = (r, g, b)

    global top_x, top_y
    if data_1 < S_MIN:
        top_y = 0
    elif data_1 > S_MAX:
        top_y = dim_y-1
    else:
        top_y = min(dim_y-1, (data_1 + S_MAX) / sensor_y)
    if data_2 < S_MIN:
        top_x = 0
    elif data_2 > S_MAX:
        top_x = dim_x-1
    else:
        top_x = min(dim_x-1, (data_2 + S_MAX) / sensor_y)
    draw_bubbles(top_x, top_y)

def calc_bubbles(a_dir):
    ''' Keyboard handler for movement debug purposes '''

    # Hox: Avoid unnecessary calculations
    if g_moving:
        return

    global top_x, top_y

    if a_dir == MOVE_UP:
        if top_y > 0:
            top_y -= 1
    elif a_dir == MOVE_DOWN:
        if top_y < dim_y-1:
            top_y += 1
    elif a_dir == MOVE_LEFT:
        if top_x > 0:
            top_x -= 1
    elif a_dir == MOVE_RIGHT:
        if top_x < dim_x-1:
            top_x += 1

    draw_bubbles(top_x, top_y)

def sensor_to_color(data):
    ''' Convert raw sensor value into 0-255 RGB value '''
    d = max(data, -300)       # Minimum -300
    d = min(d, 300)           # Maximum +300
    d = d + 300               # shift [-300,300] to [0, 600]
    return ((d * 256) / 601)  # Scale from 601 to 256 value

def get_color():
    ''' Get next RGB color '''
    # Generate random RGB color
    return (random.choice(range(0,255)), random.choice(range(0,255)), random.choice(range(0,255)))

def change_color(a_red, a_green, a_blue):
    ''' Change given color just a little bit '''
    if g_color:
        # Sensor values as RGB color
        return c_color

    # Original change color routine
    global dir_red, dir_green, dir_blue
    # Red
    value = random.choice([0, 1, 2])
    if dir_red < 0:
        value = -value
    a_red = validate_rgb(a_red + value)
    if a_red <= RGB_MIN or a_red >= RGB_MAX:
        dir_red = -dir_red
    # Green
    value = random.choice([1, 2, 3])
    if dir_green < 0:
        value = -value
    a_green = validate_rgb(a_green + value)
    if a_green <= RGB_MIN or a_green >= RGB_MAX:
        dir_green = -dir_green
    # Blue
    value = random.choice([2, 3, 4])
    if dir_blue < 0:
        value = -value
    a_blue = validate_rgb(a_blue + value)
    if a_blue <= RGB_MIN or a_blue >= RGB_MAX:
        dir_blue = -dir_blue
    return (a_red, a_green, a_blue)

def validate_rgb(a_value):
    ''' Make given value usable for RGB digit '''
    if a_value < RGB_MIN:
        a_value = RGB_MIN
    elif a_value > RGB_MAX:
        a_value = RGB_MAX
    return a_value

def handle_screensaver():
    ''' Reset inactivity timer '''
    global g_screensaver_on
    e32.reset_inactivity()
    g_screensaver_on = True

def reset_bubbles():
    global my_size, my_off, my_bg, g_quad, g_atra, g_cale, g_hex, g_shape
    my_size = orig_size
    my_off = 0
    my_bg = 2
    g_quad = 0
    g_cale = 0
    g_hex = 1
    g_color = False
    g_atra = 0
    g_shape = 0
    init_variables()
    draw_bubbles(top_x, top_y)

def size_back(a_change):
    global my_off
    my_off += a_change
    my_off = min(my_off, my_size+1)
    my_off = max(my_off, 0)
    init_variables()
    draw_bubbles(top_x, top_y)

def back_bubbles():
    global my_bg
    my_bg = (my_bg + 1) % 3
    init_variables()
    draw_bubbles(top_x, top_y)

def bubble_attraction():
    global g_atra
    if g_atra:
        g_atra = 0
    else:
        g_atra = 1
    draw_bubbles(top_x, top_y)

def screen_quad():
    global g_quad
    g_quad = (g_quad + 1) % 4
    draw_bubbles(top_x, top_y)

def screen_hex():
    global g_hex
    g_hex = (g_hex + 1) % 2
    draw_bubbles(top_x, top_y)

def change_shape():
    global g_shape
    g_shape = (g_shape + 1) % 8
    draw_bubbles(top_x, top_y)

def color_sensor():
    ''' Control if new color from random or sensor values '''
    if not SENSOR:
        return
    global g_color
    if g_color:
        g_color = False
    else:
        g_color = True

def caleidoscope():
    ''' Toggle caleidoscope mode on/off '''
    global g_cale
    g_cale = (g_cale + 1) % 4
    draw_bubbles(top_x, top_y)

def size_bubbles(a_change):
    global my_size
    my_size += a_change
    init_variables()
    draw_bubbles(top_x, top_y)

def draw_bubbles(top_x, top_y):
    ''' Show that something happened '''
    # RunTimeError: Image object busy at 140 graphics.py self._wait
    global g_moving
    if g_moving:
        return
    g_moving = True

    global rgb_red, rgb_green, rgb_blue
    (rgb_red, rgb_green, rgb_blue) = change_color(rgb_red, rgb_green, rgb_blue)

    # N82 Settings UI has minimum value 5 seconds
    # Set timeour in 4 seconds
    global g_screensaver_on
    if g_screensaver_on:
        g_screensaver_on = False
        my_timer.cancel()
        my_timer.after(4, handle_screensaver)

    # Would it be faster to call reset once a frame?
    #e32.reset_inactivity()

    # Draw new
    img.clear(RGB_BLACK)

    # One more line for partial top and bottom edges
    yy = dim_y + 2

    for j in range(yy):

        # Partial bubble line
        j = j-1

        # Corners for background colorized tile
        a2 = pnt_y = off_y + j * my_size
        b2 = a2 + my_size
        # Distance from center bubble: Pythagoras
        dj2 = (j-top_y)**2

        # Caleidoscope needs only top half
        if g_cale and not g_quad:
            if 2 * j > dim_y:
                break

        # One more line for partial left and right edges
        xx = dim_x + 2

        for i in range(xx):

            # Partial bubble line
            i = i-1

            # Caleidoscope needs only left half
            if g_cale and not g_quad:
                if 2 * i > dim_x:
                    break

            # Corners for background colorized tile
            a1 = pnt_x = off_x + i * my_size
            b1 = a1 + my_size
            pnt_y = a2

            # Hex
            if g_hex and j%2:
                a1 = pnt_x = a1 - my_size/2
                b1 = b1 - my_size/2

            # Distance from center bubble: Pythagoras
            d = int(math.sqrt((i-top_x)**2 + dj2))
            if d < 1:
                d = 1
            else:
                d *= 2
            d2 = d * 2

            if not g_atra:
                # Fix coordinates based on distance
                if i < top_x:
                    bot_x = pnt_x + my_size
                    pnt_x += d
                elif i > top_x:
                    bot_x = pnt_x + my_size - d
                else:
                    bot_x = pnt_x + my_size - d/2
                    pnt_x +=  d/2
                if j < top_y:
                    bot_y = pnt_y + my_size
                    pnt_y +=  d
                elif j > top_y:
                    bot_y = pnt_y + my_size - d
                else:
                    bot_y = pnt_y + my_size - d/2
                    pnt_y +=  d/2
            else:

                # Fix coordinates based on distance
                if i < top_x:
                    bot_x = pnt_x + my_size - d
                elif i > top_x:
                    bot_x = pnt_x + my_size
                    pnt_x = pnt_x + d
                else:
                    bot_x = pnt_x + my_size - d/2
                    pnt_x += d/2
                if j < top_y:
                    bot_y = pnt_y + my_size - d
                elif j > top_y:
                    bot_y = pnt_y + my_size
                    pnt_y = pnt_y + d
                else:
                    bot_y = pnt_y + my_size - d/2
                    pnt_y += d/2

            # Keep every bubble visible
            if pnt_x >= bot_x:
                bot_x = pnt_x + 1
            if pnt_y >= bot_y:
                bot_y = pnt_y + 1

            # More dim colors further away from top bubble
            r, g, b = rgb_red, rgb_green, rgb_blue
            d10 = d * 10
            if r < RGB_MAX2:
                r = min(r+d10, RGB_MAX)
            else:
                r = max(r-d10, RGB_MIN)
            if g < RGB_MAX2:
                g = min(g+d10, RGB_MAX)
            else:
                g = max(g-d10, RGB_MIN)
            if b < RGB_MAX2:
                b = min(b+d10, RGB_MAX)
            else:
                b = max(b-d10, RGB_MIN)

            # Reflection for ball/circle
            z = (bot_x - pnt_x) / 4

            # Draw possible bubble background
            if my_bg == 1:
                img.rectangle((a1+my_off, a2+my_off, b1-my_off, b2-my_off), fill=(255-r, 255-g, 255-b))
            elif my_bg == 2:
                img.ellipse((a1+my_off, a2+my_off, b1-my_off, b2-my_off), fill=(255-r, 255-g, 255-b))

            # Draw a single bubble
            if g_shape == 0:
                # Shape: ball
                img.ellipse((pnt_x, pnt_y, bot_x, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
            elif g_shape == 1:
                # Shape: circle
                w = 10-d
                if w < 1:
                    w = 1
                img.ellipse((pnt_x, pnt_y, bot_x, bot_y), width=w, outline=(r, g, b), fill=(rgb_red, rgb_green, rgb_blue))
            elif g_shape == 2:
                # Shape: x
                w = 12-d
                if w < 1:
                    w = 1
                if 2*w >= my_size:
                    img.point((a1 + my_size/2, a2 + my_size/2), outline=(r, g, b))
                else:
                    w2 = w/2
                    img.line((pnt_x+w2, pnt_y+w2, bot_x-w2, bot_y-w2), width=w, outline=(r, g, b))
                    img.line((pnt_x+w2, bot_y-w2, bot_x-w2, pnt_y+w2), width=w, outline=(r, g, b))
                    img.line((pnt_x+w2, pnt_y+w2, bot_x-w2, bot_y-w2), width=1, outline=(rgb_red, rgb_green, rgb_blue))
                    img.line((pnt_x+w2, bot_y-w2, bot_x-w2, pnt_y+w2), width=1, outline=(rgb_red, rgb_green, rgb_blue))
                z = (bot_x-pnt_x)/2
            elif g_shape == 3:
                # Shape: horizontal and vertical bar
                w = 12-d
                if w < 1:
                    w = 1
                c = w/2
                img.line(((pnt_x+bot_x)/2, pnt_y+c, (pnt_x+bot_x)/2, bot_y-c), width=w, outline=(r, g, b))
                img.line((pnt_x+c, (pnt_y+bot_y)/2, bot_x-c, (pnt_y+bot_y)/2), width=w, outline=(r, g, b))
                img.line(((pnt_x+bot_x)/2, pnt_y+c, (pnt_x+bot_x)/2, bot_y-c), width=1, outline=(rgb_red, rgb_green, rgb_blue))
                img.line((pnt_x+c, (pnt_y+bot_y)/2, bot_x-c, (pnt_y+bot_y)/2), width=1, outline=(rgb_red, rgb_green, rgb_blue))
                z = (bot_x-pnt_x)/2
            elif g_shape == 4:
                # Shape: horizontal or vertical bar
                w = 12-d
                if w < 1:
                    w = 1
                c = w/2
                if (i+j)%2:
                    img.line(((pnt_x+bot_x)/2, pnt_y+c, (pnt_x+bot_x)/2, bot_y-c), width=w, outline=(r, g, b))
                else:
                    img.line((pnt_x+c, (pnt_y+bot_y)/2, bot_x-c, (pnt_y+bot_y)/2), width=w, outline=(r, g, b))
                z = (bot_x-pnt_x)/2

            elif g_shape == 5:
                # Shape: triangle box
                w = 12-d
                if w < 1:
                    w = 1
                c = w/2
                if j%2:
                    if i%2:
                        img.polygon((pnt_x, bot_y, bot_x, bot_y, bot_x, pnt_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                    else:
                        img.polygon((pnt_x, pnt_y, pnt_x, bot_y, bot_x, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                else:
                    if i%2:
                        img.polygon((pnt_x, pnt_y, bot_x, pnt_y, bot_x, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                    else:
                        img.polygon((pnt_x, pnt_y, pnt_x, bot_y, bot_x, pnt_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                z = (bot_x-pnt_x)/2

            elif g_shape == 6:
                # Shape: triangles
                w = 12-d
                if w < 1:
                    w = 1
                c = w/2
                if j%2:
                    if i%2:
                        img.polygon((pnt_x, bot_y, bot_x, pnt_y, bot_x, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                    else:
                        img.polygon((pnt_x, pnt_y, bot_x, pnt_y, bot_x, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                else:
                    if i%2:
                        img.polygon((pnt_x, pnt_y, pnt_x, bot_y, bot_x, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                    else:
                        img.polygon((pnt_x, pnt_y, pnt_x, bot_y, bot_x, pnt_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                z = (bot_x-pnt_x)/2

            elif g_shape == 7:
                # Shape: 4x square
                w = 12-d
                if w < 1:
                    w = 1
                c = w
                # Must be in order: top left to bottom right
                # Rect: just replace ellipse with rectangle
                # Points
                img.ellipse((pnt_x, pnt_y, pnt_x+c, pnt_y+c), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                img.ellipse((bot_x-c, bot_y-c, bot_x, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                img.ellipse((bot_x-c, pnt_y, bot_x, pnt_y+c), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                img.ellipse((pnt_x, bot_y-c, pnt_x+c, bot_y), outline=(rgb_red, rgb_green, rgb_blue), fill=(r, g, b))
                z = (bot_x-pnt_x)/2

            # Reflection
            if d < my_size:
                w = (bot_x - pnt_x) / 4
                r = min(r+100, RGB_MAX)
                g = min(g+100, RGB_MAX)
                b = min(b+100, RGB_MAX)
                img.point((pnt_x+z, pnt_y+z), width=w, outline=(r, g, b))

    global tm1, tm4

    if g_quad:

        # Speed-up with temporary "global" variables
        # blit(image[,target=(0,0), source=((0,0),image.size), mask=None, scale=0 ])
        x, y = canvas.size
        x2 = x / 2
        y2 = y / 2

        if not tm1:
            tm1 = graphics.Image.new((x, y))
        if not tm4:
            tm4 = graphics.Image.new((x2, y2))
        tm4.blit(img, target=((0, 0), (x2, y2)), source=((off_x+1, off_y+1),(x-off_x-1, y-off_y-1)), scale=2)

        # TODO: SymbianError:[Error -4] KErroNoMemory
        # Application closed: Python ALLOC: 206246100
        # Avoid transpose, which includes memory allocation. Use blit.

        # Cornered
        if g_quad == 1:
            tm1.blit(tm4)
            tm1.blit(tm4.transpose(graphics.FLIP_LEFT_RIGHT), target=(x2, 0))
            tm1.blit(tm4.transpose(graphics.FLIP_TOP_BOTTOM), target=(0, y2))
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(x2, y2))
        # Duplicated
        elif g_quad == 2:
            tm1.blit(tm4)
            tm1.blit(tm4, target=(x2, 0))
            tm1.blit(tm4, target=(0, y2))
            tm1.blit(tm4, target=(x2, y2))
        # Opposites
        elif g_quad == 3:
            tm1.blit(tm4)
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(x2, 0))
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(0, y2))
            tm1.blit(tm4, target=(x2, y2))

        # BEWARE: Before mixed g_quad + g_cale
        # Used to be canvas.blit(tm1)
        img.blit(tm1)

    if g_cale:

        x, y = canvas.size
        x2 = x / 2
        y2 = y / 2

        global tm4_m1, tm4_m2, tm4_m3, tm4_m4
        if not tm1:
            tm1 = graphics.Image.new((x, y))
        if not tm4:
            tm4 = graphics.Image.new((x2, y2))
        if not tm4_m1:
            tm4_m1 = graphics.Image.new((x2, y2), '1')
            tm4_m1.clear(RGB_BLACK)
            tm4_m1.polygon((0,0,0,y2,x2,y2), outline=RGB_WHITE, fill=RGB_WHITE)
        if not tm4_m2:
            tm4_m2 = graphics.Image.new((x2, y2), '1')
            tm4_m2.blit(tm4_m1.transpose(graphics.ROTATE_180))
        if not tm4_m3:
            tm4_m3 = graphics.Image.new((x2, y2), '1')
            tm4_m3.blit(tm4_m1.transpose(graphics.FLIP_TOP_BOTTOM))
        if not tm4_m4:
            tm4_m4 = graphics.Image.new((x2, y2), '1')
            tm4_m4.blit(tm4_m1.transpose(graphics.FLIP_LEFT_RIGHT))

        tm4.blit(img, target=((0, 0), (x2, y2)), source=((off_x+1, off_y+1),(x-off_x-1, y-off_y-1)))

        # TODO: SymbianError:[Error -4] KErroNoMemory
        # Application closed: Python ALLOC: 206246100
        # Avoid transpose, which includes memory allocation. Use blit.

        # Four tiles
        if g_cale == 1:
            tm1.blit(tm4, mask=tm4_m1)
            tm1.blit(tm4, mask=tm4_m2)
            tm1.blit(tm4.transpose(graphics.FLIP_LEFT_RIGHT), target=(x2, 0), mask=tm4_m3)
            tm1.blit(tm4.transpose(graphics.FLIP_LEFT_RIGHT), target=(x2, 0), mask=tm4_m4)
            tm1.blit(tm4.transpose(graphics.FLIP_TOP_BOTTOM), target=(0, y2), mask=tm4_m3)
            tm1.blit(tm4.transpose(graphics.FLIP_TOP_BOTTOM), target=(0, y2), mask=tm4_m4)
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(x2, y2), mask=tm4_m1)
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(x2, y2), mask=tm4_m2)
        # Sandtimer (8 triangles)
        elif g_cale == 2:
            tm1.blit(tm4, mask=tm4_m1)
            tm1.blit(tm4.transpose(graphics.ROTATE_180), mask=tm4_m2)
            tm1.blit(tm4.transpose(graphics.FLIP_TOP_BOTTOM), target=(x2, 0), mask=tm4_m3)
            tm1.blit(tm4.transpose(graphics.FLIP_LEFT_RIGHT), target=(x2, 0), mask=tm4_m4)
            tm1.blit(tm4.transpose(graphics.FLIP_TOP_BOTTOM), target=(0, y2), mask=tm4_m3)
            tm1.blit(tm4.transpose(graphics.FLIP_LEFT_RIGHT), target=(0, y2), mask=tm4_m4)
            tm1.blit(tm4, target=(x2, y2), mask=tm4_m1)
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(x2, y2), mask=tm4_m2)
        # Diamond (8 triangles)
        elif g_cale == 3:
            tm1.blit(tm4.transpose(graphics.FLIP_TOP_BOTTOM), mask=tm4_m3)
            tm1.blit(tm4.transpose(graphics.FLIP_LEFT_RIGHT), mask=tm4_m4)
            tm1.blit(tm4, target=(x2, 0), mask=tm4_m1)
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(x2, 0), mask=tm4_m2)
            tm1.blit(tm4, target=(0, y2), mask=tm4_m1)
            tm1.blit(tm4.transpose(graphics.ROTATE_180), target=(0, y2), mask=tm4_m2)
            tm1.blit(tm4.transpose(graphics.FLIP_TOP_BOTTOM), target=(x2, y2), mask=tm4_m3)
            tm1.blit(tm4.transpose(graphics.FLIP_LEFT_RIGHT), target=(x2, y2), mask=tm4_m4)

        canvas.blit(tm1)

    else:
        # Normal case
        canvas.blit(img)

    g_moving = False

def cb_handle_redraw(dummy=(0, 0, 0, 0)):
    ''' Overwrite default screen redraw event handler '''
    if not canvas:
        return
    if img:
        canvas.blit(img)

import TopWindow
win_help = TopWindow.TopWindow()

def menu_help():
    ''' Start Help menu item '''
    global win_help
    x, y = canvas.size
    win_help.size = (x - 20, y - 40)
    win_help.position = (10,30)
    win_help.shadow = 2
    win_help.corner_type = 'corner5'
    win_help.background_color = 0xbebebe
    win_help.visible = 1

    im = graphics.Image.new((x - 30, y - 50))
    im.text((5,20),u"Rol - a - Bol", font=(None, None, appuifw.STYLE_BOLD))
    im.text((5,50),u"*  - Bigger    # - Smaller")
    im.text((5,70),u"0  - Reset")
    im.text((5,90),u"1  - Change Background")
    im.text((5,110),u"2  - Bigger Background")
    im.text((5,130),u"3  - Smaller Background")
    im.text((5,150),u"4  - Quadruple Screen")
    im.text((5,170),u"5  - Sensor Colors")
    im.text((5,190),u"6  - Hex Field")
    im.text((5,210),u"7  - Bubble Attraction")
    im.text((5,230),u"8  - Caleidoscope")
    im.text((5,250),u"9  - Change Shape")
    win_help.add_image(im, (5,5))

    win_help.show()
    capturer.start()

def menu_help_stop(dummy):
    ''' Close Help menu item '''
    win_help.hide()
    capturer.stop()

def menu_about():
    ''' Callback for menu item About '''
    appuifw.note(u'Rol-a-Bol v'+VERSION+'\n'+\
        'jouni.miettunen.googlepages.com\n(c) 2008 Jouni Miettunen')

def cb_focus(fg):
    ''' System callback to tell when focus is lost/regained '''
    global g_moving
    global g_screensaver_on
    if fg:
        # Got focus
        g_screensaver_on = True
        g_moving = False
        if SENSOR_ACC:
            sensor_acc.connect(handle_sensor_raw)
    else:
        # Lost focus
        g_screensaver_on = False
        g_moving = True
        my_timer.cancel()
        if SENSOR_ACC:
            sensor_acc.disconnect()

def cb_quit():
    ''' Do clean-up before exit '''
    my_timer.cancel()
    capturer.stop()

    # Undo bind() just in case
    canvas.bind(key_codes.EKeyLeftArrow, None)
    canvas.bind(key_codes.EKeyRightArrow, None)
    canvas.bind(key_codes.EKeyUpArrow, None)
    canvas.bind(key_codes.EKeyDownArrow, None)

    # SENSOR cleanup
    if SENSOR_ACC:
        sensor_acc.disconnect()

    app_lock.signal()

# pic_index = 0
# def take_snapshot():
#     ''' Hardcoded screen snapshot '''
#     global pic_index
#     a = graphics.screenshot()
#     filename = "c:\\python\\rolabol%02d.png" % pic_index
#     a.save(filename)
#     pic_index = (pic_index + 1) % 100

#############################################################
appuifw.app.orientation = 'portrait'
appuifw.app.screen = 'full'
appuifw.app.title = u'Rol-A-Bol'
appuifw.app.exit_key_handler = cb_quit
appuifw.app.focus = cb_focus
appuifw.app.menu = [
    (u"About", menu_about),
    (u"Help", menu_help),
    (u"Exit", cb_quit)]

canvas = appuifw.Canvas(
       redraw_callback = cb_handle_redraw)

# Calls automatically resize_callback and redraw_callback
appuifw.app.body = canvas
img = graphics.Image.new(canvas.size)

init_variables()

# Help menu item controller
capturer = keycapture.KeyCapturer(menu_help_stop)
capturer.forwarding = 0
capturer.keys = keycapture.all_keys

# Keys are used for testing the code
canvas.bind(key_codes.EKeyLeftArrow, lambda: calc_bubbles(MOVE_LEFT))
canvas.bind(key_codes.EKeyRightArrow, lambda: calc_bubbles(MOVE_RIGHT))
canvas.bind(key_codes.EKeyUpArrow, lambda: calc_bubbles(MOVE_UP))
canvas.bind(key_codes.EKeyDownArrow, lambda: calc_bubbles(MOVE_DOWN))

canvas.bind(key_codes.EKeyEnter, new_color)
canvas.bind(key_codes.EKeySelect, new_color)
canvas.bind(key_codes.EKeyStar, lambda: size_bubbles(1))
canvas.bind(key_codes.EKeyHash, lambda: size_bubbles(-1))
canvas.bind(key_codes.EKey0, reset_bubbles)
canvas.bind(key_codes.EKey1, back_bubbles)
canvas.bind(key_codes.EKey2, lambda: size_back(1))
canvas.bind(key_codes.EKey3, lambda: size_back(-1))
canvas.bind(key_codes.EKey4, screen_quad)
canvas.bind(key_codes.EKey5, color_sensor)
canvas.bind(key_codes.EKey6, screen_hex)
canvas.bind(key_codes.EKey7, bubble_attraction)
canvas.bind(key_codes.EKey8, caleidoscope)
canvas.bind(key_codes.EKey9, change_shape)

#canvas.bind(key_codes.EKeyBackspace, take_snapshot)

# SENSOR init after everything else
if SENSOR:
    # returns the dictionary of available sensors
    sensors = sensor.sensors()

    # Does this device have Accelerator Sensor
    if sensors.has_key('AccSensor'):
        SENSOR_ACC = True
        #global sensor_acc
        sensor_data = sensors['AccSensor']
        sensor_acc = sensor.Sensor(sensor_data['id'], sensor_data['category'])
        # Set my own event callback handler
        sensor_acc.connect(handle_sensor_raw)
else:
    draw_bubbles(top_x, top_y)

#############################################################
app_lock = e32.Ao_lock()
app_lock.wait()

