////////////////////////////////////////////////////////////////////////////////////////////
// KeyLock Python Extension for Symbian S60 3rd edition and higher
//
// Author: Martin Wibbels
////////////////////////////////////////////////////////////////////////////////////////////
#include <e32std.h>
#include <e32base.h>
#include <EIKBTGPC.H>
#include <avkon.hrh>
#include <aknnavi.h>
#include <aknnavide.h>
#include <eikspane.h>
#include <aknappui.h>
#include <aknkeylock.h>

#include "Python.h"
#include "symbian_python_ext_util.h"
#include "logger.h"

// PYTHON STUFF
#define keylock_type_string "keylock.KeyLockType"

#define type_KeyLock (*(PyTypeObject*)SPyGetGlobalString(keylock_type_string))

#define RETURN_PYNONE \
  Py_INCREF(Py_None);\
  return Py_None;

// type-definition
typedef struct
{
    PyObject_HEAD;
	RAknKeyLock* iKeyLock;
} obj_KeyLock;

/// Utility stuff ////////////////////////////////////////////////////////////////////////////////////////////////
static obj_KeyLock* createKeyLock(RAknKeyLock* keyLock)
{
    obj_KeyLock *lock = PyObject_New(obj_KeyLock, &type_KeyLock);

    if (lock)
    {
        lock->iKeyLock = keyLock;
    }

    return lock;
}

/// KeyLock //////////////////////////////////////////////////////////////////////////////

// ctor (factory-function)
static PyObject* keylock_GetKeyLock(PyObject* /*self*/, PyObject* /*args*/)
{
	RAknKeyLock* keyLock = new RAknKeyLock();

    TInt err = keyLock->Connect();
    if (err)
    {
        delete keyLock;
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)createKeyLock(keyLock);
}


// dtor
static void dealloc_KeyLock(obj_KeyLock* keyLock)
{
	LOG(_L("engine dealloc"));
	if (keyLock->iKeyLock) {
		keyLock->iKeyLock->Close();
		delete keyLock->iKeyLock;
		keyLock->iKeyLock = NULL;
	}
  	PyObject_Del(keyLock);
	LOG(_L("engine dealloc done"));
}

static PyObject* keylock_IsKeyLockEnabled(obj_KeyLock* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iKeyLock->IsKeyLockEnabled()?1:0);
}

static PyObject* keylock_OfferKeyLock(obj_KeyLock* self, PyObject* /*args*/)
{
	self->iKeyLock->OfferKeyLock();
	RETURN_PYNONE
}

static PyObject* keylock_EnableKeyLock(obj_KeyLock* self, PyObject* /*args*/)
{
	self->iKeyLock->EnableKeyLock();
	RETURN_PYNONE
}

static PyObject* keylock_DisableKeyLock(obj_KeyLock* self, PyObject* /*args*/)
{
	self->iKeyLock->DisableKeyLock();
	RETURN_PYNONE
}

static PyObject* keylock_EnableWithoutNote(obj_KeyLock* self, PyObject* /*args*/)
{
	self->iKeyLock->EnableWithoutNote();
	RETURN_PYNONE
}

static PyObject* keylock_DisableWithoutNote(obj_KeyLock* self, PyObject* /*args*/)
{
	self->iKeyLock->DisableWithoutNote();
	RETURN_PYNONE
}

static const PyMethodDef KeyLock_methods[] =
{
    {"IsEnabled", (PyCFunction)keylock_IsKeyLockEnabled, METH_NOARGS, "bool IsEnabled()" },
    {"Enable", (PyCFunction)keylock_EnableKeyLock, METH_NOARGS, "void Enable()" },
    {"Disable", (PyCFunction)keylock_DisableKeyLock, METH_NOARGS, "void Disable()" },
    {"Offer", (PyCFunction)keylock_OfferKeyLock, METH_NOARGS, "void Offer()" },
    {"EnableWithoutNote", (PyCFunction)keylock_EnableWithoutNote, METH_NOARGS, "void EnableWithoutNote()" },
    {"DisableWithoutNote", (PyCFunction)keylock_DisableWithoutNote, METH_NOARGS, "void DisableWithoutNote()" },
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_KeyLock(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&KeyLock_methods[0]), self, name);
}

static const PyTypeObject type_template_KeyLock =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "keylock.KeyLock",            /*tp_name*/
    sizeof(obj_KeyLock), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_KeyLock, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_KeyLock, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

static const PyMethodDef module_methods[] =
{
    {"GetKeylock", (PyCFunction)keylock_GetKeyLock, METH_NOARGS},
	{0, 0} /* sentinel */
};

static bool createType(PyTypeObject typeTemplate, char* type_string)
{
	// do type
	PyTypeObject *newTypeObject = PyObject_New(PyTypeObject, &PyType_Type);
	if (!newTypeObject)
	{
   		PyErr_SetString(PyExc_Exception, "Alloc error");
		return false;
	}

    *newTypeObject = typeTemplate;

	TInt err = SPyAddGlobalString(type_string, (PyObject*)newTypeObject);
    if (0 != err) // 0 is success
	{
        PyObject_Del(newTypeObject);
   		PyErr_SetString(PyExc_Exception, "SPyAddGlobalString failed");
        return false;
	}

    // set the type of the type object to .... type!
    newTypeObject->ob_type = &PyType_Type;

    return true;
}

DL_EXPORT(void) init_keylock()
{
  	// 3rd edition, use _keylock for separate python loader
	Py_InitModule("_keylock", (PyMethodDef*) module_methods);

	// create/register types
	if (!createType(type_template_KeyLock, keylock_type_string)) return;
}

#ifndef EKA2
GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}
#endif
