////////////////////////////////////////////////////////////////////////////////////////////
// Landmarks Python Extension for Symbian S60 3rd edition FP1 and higher
//
// Author: Martin Wibbels
// martin.wibbels@novay.nl
//
////////////////////////////////////////////////////////////////////////////////////////////
#include <e32std.h>
#include <e32base.h>
#include <EIKBTGPC.H>
#include <avkon.hrh>
#include <aknnavi.h>
#include <aknnavide.h>
#include <eikspane.h>
#include <aknappui.h>
#include <epos_cposlandmarkdatabase.h>
#include <epos_cposlmsearchcriteria.h>
#include <epos_cposlandmarkdatabase.h>
#include <epos_cposlmareacriteria.h>
#include <epos_cposlmcatnamecriteria.h>
#include <epos_cposlmcategorycriteria.h>
#include <epos_cposlmcompositecriteria.h>
#include <epos_cposlmidlistcriteria.h>
#include <epos_cposlmnearestcriteria.h>
#include <epos_cposlmtextcriteria.h>
#include <epos_cposlandmarksearch.h> 
#include <epos_cposlmoperation.h> 
#include <epos_cposlmitemiterator.h> 
#include <epos_cposlmcategorymanager.h>
#include <epos_cposlandmarkcategory.h>
#include <clmkcategoryselectordlg.h>
#include <clmklandmarkselectordlg.h>
#include <TLmkItemIdDbCombiInfo.h>		//For multi db support, combiinfo struct
#include <clmkeditordlg.h> 
#include <epos_landmarks.h>
#include <epos_cposlmdatabasemanager.h> 
#ifdef __SERIES60_31__
#include <epos_poslandmarkserialization.h>
#include <epos_poslmcategoryserialization.h>
#endif
#include <epos_cposlandmarkencoder.h>
#include <epos_cposlandmarkparser.h>

#include "Python.h"
#include "symbian_python_ext_util.h"
#include "logger.h"

// PYTHON STUFF
#define landmarkdatabase_type_string "landmarks.LandmarkDatabaseType"
#define searchcriteria_type_string "landmarks.SearchCriteriaType"
#define landmarksearch_type_string "landmarks.LandmarkSearchType"
#define lmoperation_type_string "landmarks.LmOperationType"
#define itemiterator_type_string "landmarks.ItemIteratorType"
#define landmark_type_string "landmarks.Landmark"
#define landmarkcategory_type_string "landmarks.LandmarkCategory"
#define categorymanager_type_string "landmarks.CategoryManager"
#define databasemanager_type_string "landmarks.DatabaseManager"
#define databaseinfo_type_string "landmarks.DatabaseInfo"
#define databasesettings_type_string "landmarks.DatabaseSettings"
#define landmarkencoder_type_string "landmarks.LandmarkEncoder"
#define cbufbase_type_string "landmarks.CBufBase"
#define landmarkparser_type_string "landmarks.LandmarkParser"

#define type_LandmarkDatabase (*(PyTypeObject*)SPyGetGlobalString(landmarkdatabase_type_string))
#define type_SearchCriteria (*(PyTypeObject*)SPyGetGlobalString(searchcriteria_type_string))
#define type_LandmarkSearch (*(PyTypeObject*)SPyGetGlobalString(landmarksearch_type_string))
#define type_LmOperation (*(PyTypeObject*)SPyGetGlobalString(lmoperation_type_string))
#define type_ItemIterator (*(PyTypeObject*)SPyGetGlobalString(itemiterator_type_string))
#define type_Landmark (*(PyTypeObject*)SPyGetGlobalString(landmark_type_string))
#define type_LandmarkCategory (*(PyTypeObject*)SPyGetGlobalString(landmarkcategory_type_string))
#define type_CategoryManager (*(PyTypeObject*)SPyGetGlobalString(categorymanager_type_string))
#define type_DatabaseManager (*(PyTypeObject*)SPyGetGlobalString(databasemanager_type_string))
#define type_DatabaseInfo (*(PyTypeObject*)SPyGetGlobalString(databaseinfo_type_string))
#define type_DatabaseSettings (*(PyTypeObject*)SPyGetGlobalString(databasesettings_type_string))
#define type_LandmarkEncoder (*(PyTypeObject*)SPyGetGlobalString(landmarkencoder_type_string))
#define type_CBufBase (*(PyTypeObject*)SPyGetGlobalString(cbufbase_type_string))
#define type_LandmarkParser (*(PyTypeObject*)SPyGetGlobalString(landmarkparser_type_string))

#define RETURN_PYNONE \
  Py_INCREF(Py_None);\
  return Py_None;

// type-definition
typedef struct
{
    PyObject_HEAD;
	CPosLandmarkDatabase* iDatabase;
} obj_LandmarkDatabase;

typedef struct
{
    PyObject_HEAD;
	CPosLmSearchCriteria* iSearchCriteria;
} obj_SearchCriteria;

typedef struct
{
    PyObject_HEAD;
    obj_LandmarkDatabase* iLandmarkDatabase;
	CPosLandmarkSearch* iSearch;
} obj_LandmarkSearch;

typedef struct
{
    PyObject_HEAD;
    PyObject* iParent;
	CPosLmOperation* iOperation;
} obj_LmOperation;

typedef struct
{
    PyObject_HEAD;
    PyObject* iParent;
	CPosLmItemIterator* iIterator;
} obj_ItemIterator;

typedef struct
{
    PyObject_HEAD;
	CPosLandmark* iLandmark;
} obj_Landmark;

typedef struct
{
    PyObject_HEAD;
    obj_LandmarkDatabase* iLandmarkDatabase;
	CPosLmCategoryManager* iCategoryManager;
} obj_CategoryManager;

typedef struct
{
    PyObject_HEAD;
	CPosLandmarkCategory* iLandmarkCategory;
} obj_LandmarkCategory;

typedef struct
{
    PyObject_HEAD;
	CPosLmDatabaseManager* iDatabaseManager;
} obj_DatabaseManager;

typedef struct
{
    PyObject_HEAD;
    PyObject* iParent;
	HPosLmDatabaseInfo* iDatabaseInfo;
} obj_DatabaseInfo;

typedef struct
{
    PyObject_HEAD;
    PyObject* iParent;
	TPosLmDatabaseSettings* iDatabaseSettings;
} obj_DatabaseSettings;

typedef struct
{
    PyObject_HEAD;
	CPosLandmarkEncoder* iLandmarkEncoder;
} obj_LandmarkEncoder;

typedef struct
{
    PyObject_HEAD;
	CBufBase* iBufBase;
} obj_CBufBase;

typedef struct
{
    PyObject_HEAD;
	CPosLandmarkParser* iLandmarkParser;
} obj_LandmarkParser;

class CLinkedDatabaseList {

	public:
		CLinkedDatabaseList(obj_LandmarkDatabase* landmarkDatabase);
		virtual ~CLinkedDatabaseList();
		obj_LandmarkDatabase* Find(CPosLandmarkDatabase* database);
		void setNext(CLinkedDatabaseList* next);

	private:
		obj_LandmarkDatabase* iLandmarkDatabase;
		CLinkedDatabaseList* iNext;
};

CLinkedDatabaseList::CLinkedDatabaseList(obj_LandmarkDatabase* landmarkDatabase)
{
	iLandmarkDatabase = landmarkDatabase;
	iNext = 0;
	Py_INCREF(iLandmarkDatabase);
}

CLinkedDatabaseList::~CLinkedDatabaseList()
{
	Py_DECREF(iLandmarkDatabase);
	if (iNext) delete iNext;
}

obj_LandmarkDatabase* CLinkedDatabaseList::Find(CPosLandmarkDatabase* database)
{
	if (iLandmarkDatabase->iDatabase == database) return iLandmarkDatabase;
	if (iNext) return iNext->Find(database);
	return 0;
}

void CLinkedDatabaseList::setNext(CLinkedDatabaseList* next) {
	if (iNext) iNext->setNext(next);
	iNext = next;
}

static void copyItemIdArray(PyObject* idList, RArray<TPosLmItemId>& idArray, bool& error) {
	int count = PyList_Size(idList);
	for (int i = 0; i < count; i++) 
	{
		PyObject* element = PyList_GetItem(idList, i);
		if (PyInt_Check(element)) 
		{
			idArray.Append((TPosLmItemId)PyInt_AsLong(element));
		}
		else error = true;
		Py_DECREF(element);
	}
}

static void copyUintArray(PyObject* idList, RArray<TUint>& idArray, bool& error) {
	int count = PyList_Size(idList);
	for (int i = 0; i < count; i++) 
	{
		PyObject* element = PyList_GetItem(idList, i);
		if (PyInt_Check(element)) 
		{
			idArray.Append((TUint)PyInt_AsLong(element));
		}
		else error = true;
		Py_DECREF(element);
	}
}

static PyObject* copyItemArray(RArray<TPosLmItemId>& idArray) {
	int count = idArray.Count();
	PyObject* resultList = PyList_New(count);
	for (int i = 0; i < count; i++) {
		TPosLmItemId item = idArray[i];
		PyList_SetItem(resultList, i, Py_BuildValue("i", item)); // Note: SetItem steals a reference
	}
	idArray.Close();

	return resultList;
}

static obj_ItemIterator* createIterator(PyObject* parent, CPosLmItemIterator* landmarkIter)
{
    obj_ItemIterator* itemIterator = PyObject_New(obj_ItemIterator, &type_ItemIterator);

    if (itemIterator) {
		itemIterator->iIterator = landmarkIter;
        itemIterator->iParent = parent;
		Py_INCREF(parent);
	}

	return itemIterator;
}

static obj_LmOperation* createOperation(PyObject* parent, CPosLmOperation* operation)
{
    obj_LmOperation* lmOperation = PyObject_New(obj_LmOperation, &type_LmOperation);

    if (lmOperation) {
		lmOperation->iOperation = operation;
        lmOperation->iParent = parent;
		Py_INCREF(parent);
	}

	return lmOperation;
}

static obj_LandmarkDatabase* createDatabase(CPosLandmarkDatabase* database)
{
    obj_LandmarkDatabase *landmarksDatabase = PyObject_New(obj_LandmarkDatabase, &type_LandmarkDatabase);

    if (landmarksDatabase)
    {
        landmarksDatabase->iDatabase = database;
        // check if database requires initialization, if yes do so and block
        if (landmarksDatabase->iDatabase->IsInitializingNeeded())
        {
        	ExecuteAndDeleteLD(landmarksDatabase->iDatabase->InitializeL());
        }
    }

    return landmarksDatabase;
}

static obj_DatabaseInfo* createDatabaseInfo(PyObject* parent, HPosLmDatabaseInfo* info)
{
    obj_DatabaseInfo* databaseInfo = PyObject_New(obj_DatabaseInfo, &type_DatabaseInfo);

    if (databaseInfo) {
		databaseInfo->iDatabaseInfo = info;
        databaseInfo->iParent = parent;
		if (parent) Py_INCREF(parent);
	}

	return databaseInfo;
}

static obj_DatabaseSettings* createDatabaseSettings(PyObject* parent, TPosLmDatabaseSettings* settings)
{
    obj_DatabaseSettings* databaseSettings = PyObject_New(obj_DatabaseSettings, &type_DatabaseSettings);

    if (databaseSettings) {
		databaseSettings->iDatabaseSettings = settings;
        databaseSettings->iParent = parent;
		if (parent) Py_INCREF(parent);
	}

	return databaseSettings;
}

/// LandmarkDatabase //////////////////////////////////////////////////////////////////////////////

// ctor (factory-function)
static PyObject* landmarkdatabase_OpenDefaultLandmarkDatabase(PyObject* /*self*/, PyObject* /*args*/)
{
	CPosLandmarkDatabase* database;

    TRAPD(err,
        database = CPosLandmarkDatabase::OpenL();
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)createDatabase(database);
}

static PyObject* landmarkdatabase_OpenLandmarkDatabase(PyObject* /*self*/, PyObject* args)
{
	char* uri;
	int uriLen;

	if (PyArg_ParseTuple(args, "u#:OpenLandmarkDatabase", &uri, &uriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);
		CPosLandmarkDatabase* database;

	    TRAPD(err,
	        database = CPosLandmarkDatabase::OpenL(uriDesc);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)createDatabase(database);
	}
	return 0;
}

static PyObject* landmarkdatabase_LandmarkIterator(obj_LandmarkDatabase* self, PyObject* args)
{
	int sortAttribute;
	int sortOrder;

	if (PyArg_ParseTuple(args, "ii", &sortAttribute, &sortOrder)) {
		obj_ItemIterator* itemIterator = NULL;

	    TRAPD(err,
			CPosLmItemIterator* landmarkIter;

			if (((CPosLandmark::TAttributes)sortAttribute) != CPosLandmark::ENoAttribute)
			{
				TPosLmSortPref sortPref((CPosLandmark::TAttributes)sortAttribute, (TPosLmSortPref::TSortOrder)sortOrder);
				landmarkIter = self->iDatabase->LandmarkIteratorL(sortPref);
			}
			else
			{
				landmarkIter = self->iDatabase->LandmarkIteratorL();
			}

		    itemIterator = createIterator((PyObject*)self, landmarkIter);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    return (PyObject*)itemIterator;
	}
	return 0;
}

// dtor
static void dealloc_LandmarkDatabase(obj_LandmarkDatabase* landmarksDatabase)
{
	LOG(_L("database dealloc"));
	if (landmarksDatabase->iDatabase) delete landmarksDatabase->iDatabase;
	landmarksDatabase->iDatabase = NULL;
    PyObject_Del(landmarksDatabase);
	LOG(_L("database dealloc done"));
}

static PyObject* landmarkdatabase_close(obj_LandmarkDatabase* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iDatabase) delete self->iDatabase;
		self->iDatabase = NULL;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* landmarkdatabase_CreateSearch(obj_LandmarkDatabase* landmarkDatabase, PyObject* /*args*/)
{
    obj_LandmarkSearch* landmarkSearch = PyObject_New(obj_LandmarkSearch, &type_LandmarkSearch);

    if (!landmarkSearch)
    {
        PyErr_SetString(PyExc_TypeError, "Alloc error");
        return 0;
    }

    TRAPD(err,
		landmarkSearch->iSearch = CPosLandmarkSearch::NewL(*(landmarkDatabase->iDatabase));
        landmarkSearch->iLandmarkDatabase = landmarkDatabase;
		Py_INCREF(landmarkDatabase);
    );

    if (err)
    {
        PyObject_Del(landmarkSearch);
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)landmarkSearch;
}

static PyObject* landmarkdatabase_CreateCategoryManager(obj_LandmarkDatabase* landmarkDatabase, PyObject* /*args*/)
{
    obj_CategoryManager *categoryManager = PyObject_New(obj_CategoryManager, &type_CategoryManager);

    if (!categoryManager)
    {
        PyErr_SetString(PyExc_TypeError, "Alloc error");
        return 0;
    }

    TRAPD(err,
        categoryManager->iCategoryManager = CPosLmCategoryManager::NewL(*(landmarkDatabase->iDatabase));
        categoryManager->iLandmarkDatabase = landmarkDatabase;
		Py_INCREF(landmarkDatabase);
    );

    if (err)
    {
        PyObject_Del(categoryManager);
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)categoryManager;
}

static PyObject* landmarkdatabase_AddLandmark(obj_LandmarkDatabase* self, PyObject* args)
{
	obj_Landmark* landmark;
	if (PyArg_ParseTuple(args, "O!:AddLandmark", &type_Landmark, &landmark)) {
		TPosLmItemId result;
	    TRAPD(err,
			result = self->iDatabase->AddLandmarkL(*(landmark->iLandmark));
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

		return Py_BuildValue("i", result);
	}
	return 0;
}

static PyObject* landmarkdatabase_UpdateLandmark(obj_LandmarkDatabase* self, PyObject* args)
{
	obj_Landmark* landmark;
	if (PyArg_ParseTuple(args, "O!:UpdateLandmark", &type_Landmark, &landmark)) {
	    TRAPD(err,
			self->iDatabase->UpdateLandmarkL(*(landmark->iLandmark));
	    );

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkdatabase_RemoveLandmark(obj_LandmarkDatabase* self, PyObject* args)
{
	TPosLmItemId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
	    TRAPD(err,
			self->iDatabase->RemoveLandmarkL(id);
	    );

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkdatabase_RemoveAllLandmarks(obj_LandmarkDatabase* self, PyObject* /*args*/)
{
	CPosLmOperation* operation;
    TRAPD(err,
		operation = self->iDatabase->RemoveAllLandmarksL();
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)createOperation((PyObject*)self, operation);
}

static PyObject* landmarkdatabase_ReadLandmark(obj_LandmarkDatabase* self, PyObject* args)
{
	TPosLmItemId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
		CPosLandmark* landmark;
	    TRAPD(err,
			landmark = self->iDatabase->ReadLandmarkLC(id);
			CleanupStack::Pop();
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    obj_Landmark* lm = PyObject_New(obj_Landmark, &type_Landmark);

	    if (lm) {
			lm->iLandmark = landmark;
		}

		return (PyObject*)lm;
	}
	return 0;
}

static PyObject* landmarkdatabase_ReadPartialLandmark(obj_LandmarkDatabase* self, PyObject* args)
{
	TPosLmItemId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
		CPosLandmark* landmark;
	    TRAPD(err,
			landmark = self->iDatabase->ReadPartialLandmarkLC(id);
			CleanupStack::Pop();
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    obj_Landmark* lm = PyObject_New(obj_Landmark, &type_Landmark);

	    if (lm) {
			lm->iLandmark = landmark;
		}

		return (PyObject*)lm;
	}
	return 0;
}

static PyObject* landmarkdatabase_PartialReadParameters(obj_LandmarkDatabase* self, PyObject* /*args*/)
{
	CPosLmPartialReadParameters* parameters;
    TRAPD(err,
		parameters = self->iDatabase->PartialReadParametersLC();
		CleanupStack::Pop();
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

	RArray<TUint> idArray;
	parameters->GetRequestedPositionFields(idArray);
	int count = idArray.Count();
	PyObject* resultList = PyList_New(count);
	for (int i = 0; i < count; i++) {
		TUint item = idArray[i];
		PyList_SetItem(resultList, i, Py_BuildValue("i", item)); // Note: SetItem steals a reference
	}
	CPosLandmark::TAttributes attributes = parameters->RequestedAttributes();
	delete parameters;
	idArray.Close();

	return Py_BuildValue("(iO)", attributes, resultList);
}

static PyObject* landmarkdatabase_SetPartialReadParameters(obj_LandmarkDatabase* self, PyObject* args)
{
	CPosLandmark::TAttributes attributes;
	PyObject* idList;
	if (PyArg_ParseTuple(args, "iO", &attributes, &idList)) {
	    if (!PyList_Check(idList))
		{
	        PyErr_SetString(PyExc_TypeError, "Second argument must be a list");
		    return 0;
		}

		bool error = false;
		RArray<TUint> idArray;

		copyUintArray(idList, idArray, error);

	    if (error)
	    {
	    	idArray.Close();
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

	    TRAPD(err,
			CPosLmPartialReadParameters* parameters = CPosLmPartialReadParameters::NewLC();
			parameters->SetRequestedAttributes(attributes);
			parameters->SetRequestedPositionFields(idArray);
			self->iDatabase->SetPartialReadParametersL(*parameters);
			CleanupStack::Pop();
			delete parameters;
	    );

	    idArray.Close();

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static void disableMapAndNavigation(CLmkEditorDlg *dlg) {
#ifdef __SERIES60_31__
	dlg->DisableMapAndNavigationMenuOptions();
#endif
}

static PyObject* landmarkdatabase_ShowEditDialog(obj_LandmarkDatabase* self, PyObject* args)
{
	// need: landmark id, attributes, editor mode, map&navigation menu options allowed
	TPosLmItemId id;
	TLmkEditorAttributes editorAttributes;
	TLmkEditorMode editorMode;
	int mapAndNavigationAllowed;
	if (PyArg_ParseTuple(args, "iiii", &id, &editorAttributes, &editorMode, &mapAndNavigationAllowed)) {
		CLmkEditorDlg::TLmkEditorParams editParams;
	    editParams.iAttributes = editorAttributes; 
	    editParams.iEditorMode = editorMode;

	    int dialogResult;
	    int err;
	    Py_BEGIN_ALLOW_THREADS

	    TRAP(err,
			CLmkEditorDlg *dlg = CLmkEditorDlg::NewL(*(self->iDatabase), id, editParams);
			if (!mapAndNavigationAllowed) disableMapAndNavigation(dlg);
			dialogResult = dlg->ExecuteLD();
	    );

	    Py_END_ALLOW_THREADS

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return Py_BuildValue("i", dialogResult);
	}
	return 0;
}

static PyObject* landmarkdatabase_ShowNewDialog(obj_LandmarkDatabase* self, PyObject* args)
{
	// need: landmark id, attributes, editor mode, map&navigation menu options allowed
	obj_Landmark* landmark;
	TLmkEditorAttributes editorAttributes;
	TLmkEditorMode editorMode;
	int mapAndNavigationAllowed;
	if (PyArg_ParseTuple(args, "O!iii", &type_Landmark, &landmark, &editorAttributes, &editorMode, &mapAndNavigationAllowed)) {
		CLmkEditorDlg::TLmkEditorParams editParams;
	    editParams.iAttributes = editorAttributes; 
	    editParams.iEditorMode = editorMode;

	    int dialogResult;
	    int err;
	    Py_BEGIN_ALLOW_THREADS
	    TRAP(err, {
			CLmkEditorDlg *dlg = CLmkEditorDlg::NewL(*(self->iDatabase), *(landmark->iLandmark), editParams);
			if (!mapAndNavigationAllowed) disableMapAndNavigation(dlg);
			dialogResult = dlg->ExecuteLD();
	    });
	    Py_END_ALLOW_THREADS

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return Py_BuildValue("i", dialogResult);
	}
	return 0;
}

static PyObject* landmarkdatabase_ExportLandmarks(obj_LandmarkDatabase* self, PyObject* args)
{
	PyObject* idList;
	CPosLandmarkDatabase::TTransferOptions transferOptions;
	obj_LandmarkEncoder* encoder;

	if (PyArg_ParseTuple(args, "O!Oi:ExportLandmarks", &type_LandmarkEncoder, &encoder, &idList, &transferOptions)) 
	{
	    if (!PyList_Check(idList))
		{
	        PyErr_SetString(PyExc_TypeError, "Argument must be a list");
		    return 0;
		}

		bool error = false;
		RArray<TPosLmItemId> idArray;

		copyItemIdArray(idList, idArray, error);

	    if (error)
	    {
	    	idArray.Close();
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

		CPosLmOperation* operation;
	    TRAPD(err,
    		operation = self->iDatabase->ExportLandmarksL(*(encoder->iLandmarkEncoder), idArray, transferOptions);
    	);

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static PyObject* landmarkdatabase_ImportLandmarks(obj_LandmarkDatabase* self, PyObject* args)
{
	CPosLandmarkDatabase::TTransferOptions transferOptions;
	obj_LandmarkParser* parser;

	if (PyArg_ParseTuple(args, "O!Oi:ImportLandmarks", &type_LandmarkParser, &parser, &transferOptions)) 
	{
		CPosLmOperation* operation;
	    TRAPD(err,
    		operation = self->iDatabase->ImportLandmarksL(*(parser->iLandmarkParser), transferOptions);
    	);

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static PyObject* landmarkdatabase_ImportLandmarkSet(obj_LandmarkDatabase* self, PyObject* args)
{
	PyObject* indexList;
	CPosLandmarkDatabase::TTransferOptions transferOptions;
	obj_LandmarkParser* parser;

	if (PyArg_ParseTuple(args, "O!Oi:ImportLandmarkSet", &type_LandmarkParser, &parser, &indexList, &transferOptions)) 
	{
	    if (!PyList_Check(indexList))
		{
	        PyErr_SetString(PyExc_TypeError, "Argument must be a list");
		    return 0;
		}

		bool error = false;
		RArray<TUint> indexArray;

		copyUintArray(indexList, indexArray, error);

	    if (error)
	    {
	    	indexArray.Close();
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

		CPosLmOperation* operation;
	    TRAPD(err,
    		operation = self->iDatabase->ImportLandmarksL(*(parser->iLandmarkParser), indexArray, transferOptions);
    	);

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static PyObject* landmarkdatabase_ImportedLandmarksIterator(obj_LandmarkDatabase* self, PyObject* args)
{
	obj_LmOperation* operation;
	if (PyArg_ParseTuple(args, "O!:ImportedLandmarksIterator", &type_LmOperation, &operation)) {
		CPosLmItemIterator* iterator;
	    TRAPD(err,
			iterator = self->iDatabase->ImportedLandmarksIteratorL(operation->iOperation);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    
	    return (PyObject*)createIterator((PyObject*)operation, iterator);
	}
	return 0;
}

static PyObject* landmarkdatabase_PreparePartialLandmarks(obj_LandmarkDatabase* self, PyObject* args)
{
	PyObject* idList;

	if (PyArg_ParseTuple(args, "O:PreparePartialLandmarks", &idList)) 
	{
	    if (!PyList_Check(idList))
		{
	        PyErr_SetString(PyExc_TypeError, "Argument must be a list");
		    return 0;
		}

		bool error = false;
		RArray<TPosLmItemId> idArray;

		copyItemIdArray(idList, idArray, error);

	    if (error)
	    {
	    	idArray.Close();
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

		CPosLmOperation* operation;
	    TRAPD(err,
    		operation = self->iDatabase->PreparePartialLandmarksL(idArray);
    	);

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static PyObject* landmarkdatabase_TakePreparedPartialLandmarks(obj_LandmarkDatabase* self, PyObject* args)
{
	obj_LmOperation* operation;
	if (PyArg_ParseTuple(args, "O!:TakePreparedPartialLandmarks", &type_LmOperation, &operation)) {
		CArrayPtr<CPosLandmark>* landmarkArray;
	    TRAPD(err,
			landmarkArray = self->iDatabase->TakePreparedPartialLandmarksL(operation->iOperation);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    
		int count = landmarkArray->Count();
		PyObject* resultList = PyList_New(count);
		for (int i = 0; i < count; i++) {
			CPosLandmark* item = landmarkArray->At(i);

		    obj_Landmark* lm = PyObject_New(obj_Landmark, &type_Landmark);
		    if (lm) {
				lm->iLandmark = item;
				PyList_SetItem(resultList, i, (PyObject*)lm); // Note: SetItem steals a reference
			}
		}
		landmarkArray->Reset();
		delete landmarkArray;
		
		return resultList;
	}
	return 0;
}

static const PyMethodDef LandmarkDatabase_methods[] =
{
    {"Close", (PyCFunction)landmarkdatabase_close, METH_NOARGS, "void Close()" },
    {"CreateSearch", (PyCFunction)landmarkdatabase_CreateSearch, METH_NOARGS, "LandmarkSearch CreateSearch()" },
    {"LandmarkIterator", (PyCFunction)landmarkdatabase_LandmarkIterator, METH_VARARGS, "ItemIterator LandmarkIterator(int sortAttribute, int sortOrder)" },
    {"AddLandmark", (PyCFunction)landmarkdatabase_AddLandmark, METH_VARARGS, "int AddLandmark(Landmark landmark)" },
    {"UpdateLandmark", (PyCFunction)landmarkdatabase_UpdateLandmark, METH_VARARGS, "void UpdateLandmark(Landmark landmark)" },
    {"RemoveLandmark", (PyCFunction)landmarkdatabase_RemoveLandmark, METH_VARARGS, "void RemoveLandmark(int landmarkId)" },
    {"RemoveAllLandmarks", (PyCFunction)landmarkdatabase_RemoveAllLandmarks, METH_NOARGS, "void RemoveAllLandmarks()" },
    {"ReadLandmark", (PyCFunction)landmarkdatabase_ReadLandmark, METH_VARARGS, "Landmark ReadLandmark(int landmarkId)" },
    {"ReadPartialLandmark", (PyCFunction)landmarkdatabase_ReadPartialLandmark, METH_VARARGS, "Landmark ReadPartialLandmark(int landmarkId)" },
    {"PartialReadParameters", (PyCFunction)landmarkdatabase_PartialReadParameters, METH_NOARGS, "(int, [int]) PartialReadParameters()" },
    {"SetPartialReadParameters", (PyCFunction)landmarkdatabase_SetPartialReadParameters, METH_VARARGS, "void SetPartialReadParameters(int, [int])" },
    {"CreateCategoryManager", (PyCFunction)landmarkdatabase_CreateCategoryManager, METH_NOARGS, "CategoryManager CreateCategoryManager()" },
    {"ShowEditDialog", (PyCFunction)landmarkdatabase_ShowEditDialog, METH_VARARGS, "int ShowEditDialog(int landmarkId, int attributes, int editorMode, int mapAndNavigationMenuEnabled)" },
    {"ShowNewDialog", (PyCFunction)landmarkdatabase_ShowNewDialog, METH_VARARGS, "int ShowNewDialog(Landmark landmark, int attributes, int editorMode, int mapAndNavigationMenuEnabled)" },
    {"ExportLandmarks", (PyCFunction)landmarkdatabase_ExportLandmarks, METH_VARARGS, "LmOperation ExportLandmarks(LandmarkEncoder encoder, [int] landmarkIds, int transferOptions)" },
    {"ImportLandmarkSet", (PyCFunction)landmarkdatabase_ImportLandmarkSet, METH_VARARGS, "LmOperation ImportLandmarkSet(LandmarkParser parser, [int] landmarkIds, int transferOptions)" },
    {"ImportLandmarks", (PyCFunction)landmarkdatabase_ImportLandmarks, METH_VARARGS, "LmOperation ImportLandmarks(LandmarkParser parser, int transferOptions)" },
    {"ImportedLandmarksIterator", (PyCFunction)landmarkdatabase_ImportedLandmarksIterator, METH_VARARGS, "ItemIterator ImportedLandmarksIterator(LmOperation operation)" },
    {"PreparePartialLandmarks", (PyCFunction)landmarkdatabase_PreparePartialLandmarks, METH_VARARGS, "LmOperation PreparePartialLandmarks([int] landmarkIds)" },
    {"TakePreparedPartialLandmarks", (PyCFunction)landmarkdatabase_TakePreparedPartialLandmarks, METH_VARARGS, "[Landmark] TakePreparedPartialLandmarks(LmOperation operation)" },
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_LandmarkDatabase(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&LandmarkDatabase_methods[0]), self, name);
}

static const PyTypeObject type_template_LandmarkDatabase =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.LandmarkDatabase",            /*tp_name*/
    sizeof(obj_LandmarkDatabase), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_LandmarkDatabase, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_LandmarkDatabase, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// SearchCriteria //////////////////////////////////////////////////////////////////////////////

// ctor (factory-function)
static PyObject* searchcriteria_CreateAreaCriteria(PyObject* /*self*/, PyObject* args)
{
	TReal64 southLat;
	TReal64 northLat;
	TReal64 westLong;
	TReal64 eastLong;
	if (PyArg_ParseTuple(args, "dddd:CreateAreaCriteria", &southLat, &northLat, &westLong, &eastLong)) {
	    obj_SearchCriteria* criteria = PyObject_New(obj_SearchCriteria, &type_SearchCriteria);

	    if (!criteria)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }

	    TRAPD(err,
	        criteria->iSearchCriteria = CPosLmAreaCriteria::NewLC(southLat, northLat, westLong, eastLong);
	        CleanupStack::Pop();
	    );

	    if (err)
	    {
	        PyObject_Del(criteria);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)criteria;
	}
	return 0;
}

static PyObject* searchcriteria_CreateCatNameCriteria(PyObject* /*self*/, PyObject* args)
{
	char* searchPattern;
	int searchPatternLen;

	if (PyArg_ParseTuple(args, "u#:CreateCatNameCriteria", &searchPattern, &searchPatternLen))
    {
        TPtrC searchPattern_desc((TUint16*)searchPattern, searchPatternLen);
	    obj_SearchCriteria* criteria = PyObject_New(obj_SearchCriteria, &type_SearchCriteria);

	    if (!criteria)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }

	    TRAPD(err,
	    	CPosLmCatNameCriteria* catNameCrit = CPosLmCatNameCriteria::NewLC();
	        CleanupStack::Pop();
	        criteria->iSearchCriteria = catNameCrit;
	    	catNameCrit->SetSearchPatternL(searchPattern_desc);
	    );

	    if (err)
	    {
	        PyObject_Del(criteria);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)criteria;
	}
	return 0;
}

static PyObject* searchcriteria_CreateCategoryCriteria(PyObject* /*self*/, PyObject* args)
{
	TPosLmItemId itemId;
	TPosLmGlobalCategory globalCategory;
	char* categoryName;
	int categoryNameLen;

	if (PyArg_ParseTuple(args, "iiu#:CreateCategoryCriteria", &itemId, &globalCategory, &categoryName, &categoryNameLen))
    {
	    obj_SearchCriteria* criteria = PyObject_New(obj_SearchCriteria, &type_SearchCriteria);

	    if (!criteria)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }

	    TRAPD(err,
	    	CPosLmCategoryCriteria* categoryCrit = CPosLmCategoryCriteria::NewLC();
	        CleanupStack::Pop();
	        criteria->iSearchCriteria = categoryCrit;
	    	if (itemId != KPosLmNullItemId) 
	    	{
	    		categoryCrit->SetCategoryItemId(itemId);
	    	}
	    	if (globalCategory != KPosLmNullGlobalCategory) 
	    	{
	    		categoryCrit->SetGlobalCategory(globalCategory);
	    	}
	    	if ((categoryName != NULL) && (categoryNameLen > 0)) 
	    	{
		        TPtrC categoryName_desc((TUint16*)categoryName, categoryNameLen);
	    		categoryCrit->SetCategoryNameL(categoryName_desc);
	    	}
	    );

	    if (err)
	    {
	        PyObject_Del(criteria);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)criteria;
	}
	return 0;
}

static PyObject* searchcriteria_CreateCompositeCriteria(PyObject* /*self*/, PyObject* args)
{
	PyObject* childElementArray;
	int compositionType;

	if (PyArg_ParseTuple(args, "iO:CreateCompositeCriteria", &compositionType, &childElementArray))
    {
	    if (!PyList_Check(childElementArray))
		{
	        PyErr_SetString(PyExc_TypeError, "Second argument must be a list");
		    return 0;
		}
		
	    obj_SearchCriteria* criteria = PyObject_New(obj_SearchCriteria, &type_SearchCriteria);

	    if (!criteria)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }

	    TRAPD(err,
	    	CPosLmCompositeCriteria* compositeCrit = CPosLmCompositeCriteria::NewLC((CPosLmCompositeCriteria::TCompositionType)compositionType);
	        CleanupStack::Pop();
	        criteria->iSearchCriteria = compositeCrit;
			int count = PyList_Size(childElementArray);
			for (int i = 0; i < count; i++) 
			{
				obj_SearchCriteria* element = (obj_SearchCriteria*)PyList_GetItem(childElementArray, i);
				if (compositeCrit->AddArgument(element->iSearchCriteria) == KErrNone) 
				{
					// composite has taken over ownership, so remove handle from element
					element->iSearchCriteria= 0;
				}	
				Py_DECREF(element);
			}
	    );

	    if (err)
	    {
	        PyObject_Del(criteria);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)criteria;
	}
	return 0;
}

static PyObject* searchcriteria_CreateIdListCriteria(PyObject* /*self*/, PyObject* args)
{
	PyObject* idList;

	if (PyArg_ParseTuple(args, "O:CreateIdListCriteria", &idList))
    {
	    if (!PyList_Check(idList))
		{
	        PyErr_SetString(PyExc_TypeError, "Argument must be a list");
		    return 0;
		}

	    obj_SearchCriteria* criteria = PyObject_New(obj_SearchCriteria, &type_SearchCriteria);

	    if (!criteria)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }

		bool error = false;
		RArray<TPosLmItemId> idArray;

		copyItemIdArray(idList, idArray, error);

	    if (error)
	    {
	    	idArray.Close();
	        PyObject_Del(criteria);
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

	    TRAPD(err,
	    	CPosLmIdListCriteria* idListCrit = CPosLmIdListCriteria::NewLC();
	        CleanupStack::Pop();
	        criteria->iSearchCriteria = idListCrit;
	        
	        idListCrit->SetLandmarkIdsL(idArray);
	    );

    	idArray.Close();

	    if (err)
	    {
	        PyObject_Del(criteria);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)criteria;
	}
	return 0;
}

static PyObject* searchcriteria_CreateNearestCriteria(PyObject* /*self*/, PyObject* args)
{
	TReal64 latitude;
	TReal64 longitude;
	TReal32 altitude;
	int useCoverageRadius;
	TReal32 distance;

	if (PyArg_ParseTuple(args, "ddfif:CreateNearestCriteria", &latitude, &longitude, &altitude, &useCoverageRadius, &distance))
    {
	    obj_SearchCriteria* criteria = PyObject_New(obj_SearchCriteria, &type_SearchCriteria);
	    
	    TCoordinate coordinate(latitude, longitude, altitude);
	    if (!criteria)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }

	    TRAPD(err,
	    	CPosLmNearestCriteria* nearestCrit = CPosLmNearestCriteria::NewLC(coordinate, useCoverageRadius != 0);
	        CleanupStack::Pop();
	        criteria->iSearchCriteria = nearestCrit;
	    	if (distance >= 0) nearestCrit->SetMaxDistance(distance);
	    );

	    if (err)
	    {
	        PyObject_Del(criteria);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)criteria;
	}
	return 0;
}

static PyObject* searchcriteria_CreateTextCriteria(PyObject* /*self*/, PyObject* args)
{
	char* text;
	int textLen;
	int attributes;
	PyObject* fieldsList;

	if (PyArg_ParseTuple(args, "u#iO:CreateTextCriteria", &text, &textLen, &attributes, &fieldsList))
    {
	    if (!PyList_Check(fieldsList))
		{
	        PyErr_SetString(PyExc_TypeError, "Third argument must be a list");
		    return 0;
		}

	    obj_SearchCriteria* criteria = PyObject_New(obj_SearchCriteria, &type_SearchCriteria);

	    if (!criteria)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }

	    TPtrC text_desc((TUint16*)text, textLen);

		bool error = false;
		RArray<TUint> fieldArray;

		copyUintArray(fieldsList, fieldArray, error);
		
	    if (error)
	    {
	    	fieldArray.Close();
	        PyObject_Del(criteria);
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

	    TRAPD(err,
	    	CPosLmTextCriteria* textCrit = CPosLmTextCriteria::NewLC();
	        CleanupStack::Pop();
	        criteria->iSearchCriteria = textCrit;
	        
	        textCrit->SetTextL(text_desc);
	        textCrit->SetPositionFieldsToSearchL(fieldArray);
	        textCrit->SetAttributesToSearch((CPosLandmark::TAttributes)attributes);
	    );

    	fieldArray.Close();

	    if (err)
	    {
	        PyObject_Del(criteria);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)criteria;
	}
	return 0;
}

static PyObject* searchcriteria_Close(obj_SearchCriteria* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iSearchCriteria) delete self->iSearchCriteria;
		self->iSearchCriteria = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

// dtor
static void dealloc_SearchCriteria(obj_SearchCriteria* criteria)
{
	if (criteria->iSearchCriteria) delete criteria->iSearchCriteria;
	criteria->iSearchCriteria = 0;
    PyObject_Del(criteria);
}


static const PyMethodDef SearchCriteria_methods[] =
{
    {"Close", (PyCFunction)searchcriteria_Close, METH_NOARGS, "void Close()"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_SearchCriteria(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&SearchCriteria_methods[0]), self, name);
}

static const PyTypeObject type_template_SearchCriteria =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.SearchCriteria",            /*tp_name*/
    sizeof(obj_SearchCriteria), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_SearchCriteria, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_SearchCriteria, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// Landmark search ///////////////////////////////////////////////////////////////////////////////////////////

// dtor
static void dealloc_LandmarkSearch(obj_LandmarkSearch* landmarkSearch)
{
	Py_DECREF(landmarkSearch->iLandmarkDatabase);
	if (landmarkSearch->iSearch) delete landmarkSearch->iSearch;
	landmarkSearch->iSearch = NULL;
    PyObject_Del(landmarkSearch);
}

static PyObject* landmarksearch_Close(obj_LandmarkSearch* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iSearch) delete self->iSearch;
		self->iSearch = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* landmarksearch_SetMaxNumOfMatches(obj_LandmarkSearch* self, PyObject* args)
{
	int maxNum;

	if (PyArg_ParseTuple(args, "i", &maxNum)) {
		self->iSearch->SetMaxNumOfMatches(maxNum);
	    RETURN_PYNONE;
	}
	return 0;
}

static PyObject* landmarksearch_StartLandmarkSearch(obj_LandmarkSearch* self, PyObject* args)
{
	obj_SearchCriteria* searchCriteria;
	int sortAttribute;
	int sortOrder;
	int searchOnlyPreviousMatches;

	if (PyArg_ParseTuple(args, "O!iii", &type_SearchCriteria, &searchCriteria, &sortAttribute, &sortOrder, &searchOnlyPreviousMatches )) {

		obj_LmOperation* lmOperation = NULL;

        TRAPD(err,

			CPosLmOperation* operation;
			if (((CPosLandmark::TAttributes)sortAttribute) != CPosLandmark::ENoAttribute)
			{
				TPosLmSortPref sortPref((CPosLandmark::TAttributes)sortAttribute, (TPosLmSortPref::TSortOrder)sortOrder);
				operation = self->iSearch->StartLandmarkSearchL(*(searchCriteria->iSearchCriteria), sortPref, searchOnlyPreviousMatches != 0);
			}
			else
			{
				operation = self->iSearch->StartLandmarkSearchL(*(searchCriteria->iSearchCriteria), searchOnlyPreviousMatches != 0);
			}

			lmOperation = createOperation((PyObject*)self, operation);
        );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)lmOperation;
	}
	return 0;
}

static PyObject* landmarksearch_StartCategorySearch(obj_LandmarkSearch* self, PyObject* args)
{
	obj_SearchCriteria* searchCriteria;
	int sortOrder;
	int searchOnlyPreviousMatches;

	if (PyArg_ParseTuple(args, "O!ii", &type_SearchCriteria, &searchCriteria, &sortOrder, &searchOnlyPreviousMatches )) {

		obj_LmOperation* lmOperation = NULL;

        TRAPD(err,

			CPosLmOperation* operation = self->iSearch->StartCategorySearchL(*(searchCriteria->iSearchCriteria), (CPosLmCategoryManager::TCategorySortPref)sortOrder, searchOnlyPreviousMatches != 0);

		    lmOperation = createOperation((PyObject*)self, operation);
        );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)lmOperation;
	}
	return 0;
}

static PyObject* landmarksearch_MaxNumOfMatches(obj_LandmarkSearch* self, PyObject* /*args*/) {
	return Py_BuildValue("i", self->iSearch->MaxNumOfMatches());
}

static PyObject* landmarksearch_NumOfMatches(obj_LandmarkSearch* self, PyObject* /*args*/) {
	return Py_BuildValue("i", self->iSearch->NumOfMatches());
}

static PyObject* landmarksearch_MatchIterator(obj_LandmarkSearch* self, PyObject* /*args*/)
{
	obj_ItemIterator* itemIterator = NULL;

    TRAPD(err,

		CPosLmItemIterator* matchIter = self->iSearch->MatchIteratorL();

	    itemIterator = createIterator((PyObject*)self, matchIter);
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }
    return (PyObject*)itemIterator;
}

static const PyMethodDef LandmarkSearch_methods[] =
{
    {"Close", (PyCFunction)landmarksearch_Close, METH_NOARGS, "void Close()"},
    {"MaxNumOfMatches", (PyCFunction)landmarksearch_MaxNumOfMatches, METH_NOARGS, "int MaxNumOfMatches()"},
    {"SetMaxNumOfMatches", (PyCFunction)landmarksearch_SetMaxNumOfMatches, METH_VARARGS, "void SetMaxNumOfMatches(int maxNum)" },
    {"StartLandmarkSearch", (PyCFunction)landmarksearch_StartLandmarkSearch, METH_VARARGS, "LmOperation StartLandmarkSearch(SearchCritera citeria, int sortAttribute, int sortOrder, int searchOnlyPreviousMatches)" },
    {"StartCategorySearch", (PyCFunction)landmarksearch_StartCategorySearch, METH_VARARGS, "LmOperation StartCategorySearch(SearchCritera citeria, int sortOrder, int searchOnlyPreviousMatches)" },
    {"NumOfMatches", (PyCFunction)landmarksearch_NumOfMatches, METH_NOARGS, "int NumOfMatches()"},
    {"MatchIterator", (PyCFunction)landmarksearch_MatchIterator, METH_NOARGS, "ItemIterator MatchIterator()"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_LandmarkSearch(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&LandmarkSearch_methods[0]), self, name);
}

static const PyTypeObject type_template_LandmarkSearch =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.LandmarkSearch",            /*tp_name*/
    sizeof(obj_LandmarkSearch), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_LandmarkSearch, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_LandmarkSearch, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// LmOperation ///////////////////////////////////////////////////////////////////////////////////////////

// dtor
static void dealloc_LmOperation(obj_LmOperation* self)
{
	Py_DECREF(self->iParent);
	if (self->iOperation) delete self->iOperation;
	self->iOperation = NULL;
    PyObject_Del(self);
}

static PyObject* lmoperation_Cancel(obj_LmOperation* self, PyObject* /*args*/)
{
	if (self->iOperation) delete self->iOperation;
	self->iOperation = 0;
    RETURN_PYNONE;
}

static PyObject* lmoperation_NextStep(obj_LmOperation* self, PyObject* /*args*/)
{
	TRequestStatus status;
	TReal32 progress;
	self->iOperation->NextStep(status, progress);
	User::WaitForRequest(status);

	if (status == KPosLmOperationNotComplete)
	{
		return Py_BuildValue("d", progress);
	}
	else if (status == KErrNone)
	{
		progress = 2.00;
		return Py_BuildValue("d", progress);
	}
	else
	{
        return SPyErr_SetFromSymbianOSErr(status.Int());
	}
}

static PyObject* lmoperation_Execute(obj_LmOperation* self, PyObject* /*args*/)
{
	TRAPD(err,
		self->iOperation->ExecuteL();
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static const PyMethodDef LmOperation_methods[] =
{
    {"NextStep", (PyCFunction)lmoperation_NextStep, METH_NOARGS, "double NextStep()"},
    {"Execute", (PyCFunction)lmoperation_Execute, METH_NOARGS, "void Execute()"},
    {"Cancel", (PyCFunction)lmoperation_Cancel, METH_NOARGS, "void Cancel()"},
    {"Close", (PyCFunction)lmoperation_Cancel, METH_NOARGS, "void Close()"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_LmOperation(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&LmOperation_methods[0]), self, name);
}

static const PyTypeObject type_template_LmOperation =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.LmOperation",            /*tp_name*/
    sizeof(obj_LmOperation), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_LmOperation, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_LmOperation, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// ItemIterator ///////////////////////////////////////////////////////////////////////////////////////////

// dtor
static void dealloc_ItemIterator(obj_ItemIterator* self)
{
	Py_DECREF(self->iParent);
	if (self->iIterator) delete self->iIterator;
	self->iIterator = NULL;
    PyObject_Del(self);
}

static PyObject* itemiterator_Next(obj_ItemIterator* self, PyObject* /*args*/)
{
	TPosLmItemId result;
	TRAPD(err,
		result = self->iIterator->NextL();
	);
    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

	return Py_BuildValue("i", result);
}

static PyObject* itemiterator_Reset(obj_ItemIterator* self, PyObject* /*args*/)
{
	self->iIterator->Reset();
    RETURN_PYNONE;
}

static PyObject* itemiterator_NumOfItems(obj_ItemIterator* self, PyObject* /*args*/)
{
	TUint result;
	TRAPD(err,
		result = self->iIterator->NumOfItemsL();
	);
    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

	return Py_BuildValue("i", result);
}

static PyObject* itemiterator_GetItemIds(obj_ItemIterator* self, PyObject* args)
{
	TInt index;
	TInt count;
	if (PyArg_ParseTuple(args, "ii", &index, &count)) {
		RArray<TPosLmItemId> idArray;
		TRAPD(err,
			self->iIterator->GetItemIdsL(idArray, index, count);
		);
	    if (err)
	    {
	    	idArray.Close();
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

		// build result list
		return copyItemArray(idArray);
	}
	return 0;
}

static PyObject* itemiterator_Close(obj_ItemIterator* self, PyObject* /*args*/)
{
	if (self->iIterator) delete self->iIterator;
	self->iIterator = 0;
    RETURN_PYNONE;
}

static const PyMethodDef ItemIterator_methods[] =
{
    {"Next", (PyCFunction)itemiterator_Next, METH_NOARGS, "int Next()"},
    {"Reset", (PyCFunction)itemiterator_Reset, METH_NOARGS, "void Reset()"},
    {"NumOfItems", (PyCFunction)itemiterator_NumOfItems, METH_NOARGS, "int NumOfItems()"},
    {"GetItemIds", (PyCFunction)itemiterator_GetItemIds, METH_VARARGS, "[int] GetItemIds(int index, int count)"},
    {"Close", (PyCFunction)itemiterator_Close, METH_NOARGS, "void Close()"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_ItemIterator(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&ItemIterator_methods[0]), self, name);
}

static const PyTypeObject type_template_ItemIterator =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.ItemIterator",            /*tp_name*/
    sizeof(obj_ItemIterator), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_ItemIterator, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_ItemIterator, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// Landmark ///////////////////////////////////////////////////////////////////////////////////////////

// ctor (factory-function)
static PyObject* landmark_CreateLandmark(PyObject* /*self*/, PyObject* /*args*/)
{
    obj_Landmark *landmark = PyObject_New(obj_Landmark, &type_Landmark);

    if (!landmark)
    {
        PyErr_SetString(PyExc_TypeError, "Alloc error");
        return 0;
    }

    TRAPD(err,
        landmark->iLandmark = CPosLandmark::NewL();
    );

    if (err)
    {
        PyObject_Del(landmark);
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)landmark;
}

// dtor
static void dealloc_Landmark(obj_Landmark* self)
{
	if (self->iLandmark) delete self->iLandmark;
	self->iLandmark = NULL;
    PyObject_Del(self);
}

static PyObject* landmark_Close(obj_Landmark* self, PyObject* /*args*/)
{
	if (self->iLandmark) delete self->iLandmark;
	self->iLandmark = 0;
    RETURN_PYNONE;
}

static PyObject* landmark_IsPartial(obj_Landmark* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmark->IsPartial());
}

static PyObject* landmark_GetLandmarkName(obj_Landmark* self, PyObject* /*args*/)
{
	TPtrC name;
	int err = self->iLandmark->GetLandmarkName(name);
	if (err) {
		// don't throw exception, just return None
		RETURN_PYNONE
	}
	return Py_BuildValue("u#", name.Ptr(), name.Length());
}

static PyObject* landmark_SetLandmarkName(obj_Landmark* self, PyObject* args)
{
	char* name;
	int nameLen;
	if (PyArg_ParseTuple(args, "u#", &name, &nameLen)) {
        TPtrC nameDesc((TUint16*)name, nameLen);
		TRAPD(err,
			self->iLandmark->SetLandmarkNameL(nameDesc);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmark_GetLandmarkDescription(obj_Landmark* self, PyObject* /*args*/)
{
	TPtrC name;
	int err = self->iLandmark->GetLandmarkDescription(name);
	if (err) {
		// don't throw exception, just return None
		RETURN_PYNONE
	}
	return Py_BuildValue("u#", name.Ptr(), name.Length());
}

static PyObject* landmark_SetLandmarkDescription(obj_Landmark* self, PyObject* args)
{
	char* name;
	int nameLen;
	if (PyArg_ParseTuple(args, "u#", &name, &nameLen)) {
        TPtrC nameDesc((TUint16*)name, nameLen);
		TRAPD(err,
			self->iLandmark->SetLandmarkDescriptionL(nameDesc);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmark_SetPosition(obj_Landmark* self, PyObject* args)
{
	TReal64 latitude;
	TReal64 longitude;
	TReal64 altitude;
	TReal64 horAccuracy;
	TReal64 verAccuracy;
	if (PyArg_ParseTuple(args, "ddddd", &latitude, &longitude, &altitude, &horAccuracy, &verAccuracy)) {
		TCoordinate coordinate(latitude, longitude, altitude);
		TLocality locality(coordinate, horAccuracy, verAccuracy);
		TRAPD(err,
			self->iLandmark->SetPositionL(locality);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmark_GetPosition(obj_Landmark* self, PyObject* /*args*/)
{
	TLocality locality;
	int err = self->iLandmark->GetPosition(locality);
	if (err) {
		// don't throw exception, just return None
		RETURN_PYNONE
	}
	return Py_BuildValue("(ddddd)", locality.Latitude(), locality.Longitude(), locality.Altitude(), locality.HorizontalAccuracy(), locality.VerticalAccuracy());
}

static PyObject* landmark_SetCoverageRadius(obj_Landmark* self, PyObject* args)
{
	TReal64 radius;
	if (PyArg_ParseTuple(args, "d", &radius)) {
		self->iLandmark->SetCoverageRadius(radius);
	    RETURN_PYNONE;
	}
	return 0;
}

static PyObject* landmark_GetCoverageRadius(obj_Landmark* self, PyObject* /*args*/)
{
	TReal32 radius;
	int err = self->iLandmark->GetCoverageRadius(radius);
	if (err) {
		// don't throw exception, just return None
		RETURN_PYNONE
	}
	return Py_BuildValue("d", radius);
}

static PyObject* landmark_LandmarkId(obj_Landmark* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmark->LandmarkId());
}

static PyObject* landmark_AddCategory(obj_Landmark* self, PyObject* args)
{
	TPosLmItemId catId;
	if (PyArg_ParseTuple(args, "i", &catId)) {
		TRAPD(err,
			self->iLandmark->AddCategoryL(catId);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmark_RemoveCategory(obj_Landmark* self, PyObject* args)
{
	TPosLmItemId catId;
	if (PyArg_ParseTuple(args, "i", &catId)) {
		self->iLandmark->RemoveCategory(catId);
	    RETURN_PYNONE;
	}
	return 0;
}

static PyObject* landmark_GetCategories(obj_Landmark* self, PyObject* /*args*/)
{
	RArray<TPosLmItemId> idArray;
	TRAPD(err,
		self->iLandmark->GetCategoriesL(idArray);
	);
    if (err)
    {
    	idArray.Close();
        return SPyErr_SetFromSymbianOSErr(err);
    }

	// build result list
	return copyItemArray(idArray);
}

static PyObject* landmark_IsPositionFieldAvailable(obj_Landmark* self, PyObject* args)
{
	TPositionFieldId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
    	return Py_BuildValue("i", self->iLandmark->IsPositionFieldAvailable(id));
	}
	return 0;
}

static PyObject* landmark_NumOfAvailablePositionFields(obj_Landmark* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmark->NumOfAvailablePositionFields());
}

static PyObject* landmark_NextPositionFieldId(obj_Landmark* self, PyObject* args)
{
	TPositionFieldId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
    	return Py_BuildValue("i", self->iLandmark->NextPositionFieldId(id));
	}
	return 0;
}

static PyObject* landmark_FirstPositionFieldId(obj_Landmark* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmark->FirstPositionFieldId());
}

static PyObject* landmark_GetPositionField(obj_Landmark* self, PyObject* args)
{
	TPositionFieldId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
		TPtrC name;
		int err = self->iLandmark->GetPositionField(id, name);
		if (err) {
			// don't throw exception, just return None
			RETURN_PYNONE
		}
    	return Py_BuildValue("u#", name.Ptr(), name.Length());
	}
	return 0;
}

static PyObject* landmark_SetPositionField(obj_Landmark* self, PyObject* args)
{
	TPositionFieldId id;
	char* name;
	int nameLen;
	if (PyArg_ParseTuple(args, "iu#", &id, &name, &nameLen)) {
        TPtrC nameDesc((TUint16*)name, nameLen);
		TRAPD(err,
			self->iLandmark->SetPositionFieldL(id, nameDesc);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmark_RemovePositionField(obj_Landmark* self, PyObject* args)
{
	TPositionFieldId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
		self->iLandmark->RemovePositionField(id);
	    RETURN_PYNONE;
	}
	return 0;
}

static PyObject* landmark_SetIcon(obj_Landmark* self, PyObject* args)
{
	char* name;
	int nameLen;
	int iconIndex;
	int iconMaskIndex;
	if (PyArg_ParseTuple(args, "u#ii", &name, &nameLen, &iconIndex, &iconMaskIndex)) {
        TPtrC nameDesc((TUint16*)name, nameLen);
		TRAPD(err,
			self->iLandmark->SetIconL(nameDesc, iconIndex, iconMaskIndex);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmark_GetIcon(obj_Landmark* self, PyObject* /*args*/)
{
	int iconIndex;
	int iconMaskIndex;
    TPtrC nameDesc;
	int err = self->iLandmark->GetIcon(nameDesc, iconIndex, iconMaskIndex);
	if (err) {
		// don't throw exception, just return None
		RETURN_PYNONE
	}
	return Py_BuildValue("u#ii", nameDesc.Ptr(), nameDesc.Length(), iconIndex, iconMaskIndex);
}

static PyObject* landmark_RemoveLandmarkAttributes(obj_Landmark* self, PyObject* args)
{
	CPosLandmark::TAttributes id;
	if (PyArg_ParseTuple(args, "i", &id)) {
		self->iLandmark->RemoveLandmarkAttributes(id);
	    RETURN_PYNONE;
	}
	return 0;
}

static const PyMethodDef Landmark_methods[] =
{
    {"Close", (PyCFunction)landmark_Close, METH_NOARGS, "void Close()"},
    {"IsPartial", (PyCFunction)landmark_IsPartial, METH_NOARGS, "boolean IsPartial()"},
    {"GetLandmarkName", (PyCFunction)landmark_GetLandmarkName, METH_NOARGS, "string GetLandmarkName()"},
    {"SetLandmarkName", (PyCFunction)landmark_SetLandmarkName, METH_VARARGS, "void SetLandmarkName(string name)"},
    {"GetPosition", (PyCFunction)landmark_GetPosition, METH_NOARGS, "(lat, long, alt, horAcc, verAcc) GetPosition()"},
    {"SetPosition", (PyCFunction)landmark_SetPosition, METH_VARARGS, "void SetPosition(double lat, double long, double alt, double horAcc, double verAcc)"},
    {"GetCoverageRadius", (PyCFunction)landmark_GetCoverageRadius, METH_NOARGS, "double GetCoverageRadius()"},
    {"SetCoverageRadius", (PyCFunction)landmark_SetCoverageRadius, METH_VARARGS, "void SetCoverageRadius(double radius)"},
    {"LandmarkId", (PyCFunction)landmark_LandmarkId, METH_NOARGS, "int LandmarkId()"},
    {"GetLandmarkDescription", (PyCFunction)landmark_GetLandmarkDescription, METH_NOARGS, "string GetLandmarkDescription()"},
    {"SetLandmarkDescription", (PyCFunction)landmark_SetLandmarkDescription, METH_VARARGS, "void SetLandmarkDescription(string name)"},
    {"AddCategory", (PyCFunction)landmark_AddCategory, METH_VARARGS, "void AddCategory(int categoryId)"},
    {"RemoveCategory", (PyCFunction)landmark_RemoveCategory, METH_VARARGS, "void RemoveCategory(int categoryId)"},
    {"GetCategories", (PyCFunction)landmark_GetCategories, METH_NOARGS, "[int] GetCategories()"},
    {"IsPositionFieldAvailable", (PyCFunction)landmark_IsPositionFieldAvailable, METH_VARARGS, "boolean IsPositionFieldAvailable(int fieldId)"},
    {"NumOfAvailablePositionFields", (PyCFunction)landmark_NumOfAvailablePositionFields, METH_NOARGS, "int NumOfAvailablePositionFields()"},
    {"NextPositionFieldId", (PyCFunction)landmark_NextPositionFieldId, METH_VARARGS, "int NextPositionFieldId(int fieldId)"},
    {"FirstPositionFieldId", (PyCFunction)landmark_FirstPositionFieldId, METH_NOARGS, "int FirstPositionFieldId()"},
    {"GetPositionField", (PyCFunction)landmark_GetPositionField, METH_VARARGS, "string GetPositionField(int fieldId)"},
    {"SetPositionField", (PyCFunction)landmark_SetPositionField, METH_VARARGS, "void SetPositionField(int fieldId, string name)"},
    {"RemovePositionField", (PyCFunction)landmark_RemovePositionField, METH_VARARGS, "void RemovePositionField(int fieldId)"},
    {"GetIcon", (PyCFunction)landmark_GetIcon, METH_VARARGS, "(iconFileName, iconIndex, iconMaskIndex) GetIcon()"},
    {"SetIcon", (PyCFunction)landmark_SetIcon, METH_VARARGS, "void SetIcon(iconFileName, iconIndex, iconMaskIndex)"},
    {"RemoveLandmarkAttributes", (PyCFunction)landmark_RemoveLandmarkAttributes, METH_VARARGS, "void RemoveLandmarkAttributes(int attributes)"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_Landmark(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&Landmark_methods[0]), self, name);
}

static const PyTypeObject type_template_Landmark =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.Landmark",            /*tp_name*/
    sizeof(obj_Landmark), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_Landmark, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_Landmark, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// CategoryManager ///////////////////////////////////////////////////////////////////////////////////////////

// dtor
static void dealloc_CategoryManager(obj_CategoryManager* self)
{
	if (self->iCategoryManager) delete self->iCategoryManager;
	Py_DECREF(self->iLandmarkDatabase);
	self->iLandmarkDatabase = NULL;
    PyObject_Del(self);
}

static PyObject* categorymanager_Close(obj_CategoryManager* self, PyObject* /*args*/)
{
	if (self->iCategoryManager) delete self->iCategoryManager;
	self->iCategoryManager = 0;
    RETURN_PYNONE;
}

static PyObject* categorymanager_CategoryIterator(obj_CategoryManager* self, PyObject* args)
{
	int sortOrder;

	if (PyArg_ParseTuple(args, "i", &sortOrder)) {
		obj_ItemIterator* itemIterator = NULL;

	    TRAPD(err,
			CPosLmItemIterator* categoryIter = self->iCategoryManager->CategoryIteratorL((CPosLmCategoryManager::TCategorySortPref)sortOrder);

		    itemIterator = createIterator((PyObject*)self, categoryIter);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    return (PyObject*)itemIterator;
	}
	return 0;
}

static PyObject* categorymanager_ReferencedCategoryIterator(obj_CategoryManager* self, PyObject* args)
{
	int sortOrder;

	if (PyArg_ParseTuple(args, "i", &sortOrder)) {
		obj_ItemIterator* itemIterator = NULL;

	    TRAPD(err,
			CPosLmItemIterator* categoryIter = self->iCategoryManager->ReferencedCategoryIteratorL((CPosLmCategoryManager::TCategorySortPref)sortOrder);

		    itemIterator = createIterator((PyObject*)self, categoryIter);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    return (PyObject*)itemIterator;
	}
	return 0;
}

static PyObject* categorymanager_ReadCategory(obj_CategoryManager* self, PyObject* args)
{
	TPosLmItemId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
		CPosLandmarkCategory* category;
	    TRAPD(err,
			category = self->iCategoryManager->ReadCategoryLC(id);
			CleanupStack::Pop();
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    obj_LandmarkCategory* lmc = PyObject_New(obj_LandmarkCategory, &type_LandmarkCategory);

	    if (lmc) {
			lmc->iLandmarkCategory = category;
		}

		return (PyObject*)lmc;
	}
	return 0;
}

static PyObject* categorymanager_AddCategory(obj_CategoryManager* self, PyObject* args)
{
	obj_LandmarkCategory* category;
	if (PyArg_ParseTuple(args, "O!:AddCategory", &type_LandmarkCategory, &category)) {
		TPosLmItemId result;
	    TRAPD(err,
			result = self->iCategoryManager->AddCategoryL(*(category->iLandmarkCategory));
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

		return Py_BuildValue("i", result);
	}
	return 0;
}

static PyObject* categorymanager_UpdateCategory(obj_CategoryManager* self, PyObject* args)
{
	obj_LandmarkCategory* category;
	if (PyArg_ParseTuple(args, "O!:UpdateCategory", &type_LandmarkCategory, &category)) {
	    TRAPD(err,
			self->iCategoryManager->UpdateCategoryL(*(category->iLandmarkCategory));
	    );

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* categorymanager_RemoveCategory(obj_CategoryManager* self, PyObject* args)
{
	TPosLmItemId id;
	if (PyArg_ParseTuple(args, "i", &id)) {
	    TRAPD(err,
			self->iCategoryManager->RemoveCategoryL(id);
	    );

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* categorymanager_GetCategory(obj_CategoryManager* self, PyObject* args)
{
	char* categoryName;
	int categoryNameLen;
	if (PyArg_ParseTuple(args, "u#", &categoryName, &categoryNameLen)) {
        TPtrC categoryNameDesc((TUint16*)categoryName, categoryNameLen);

		TPosLmItemId id;
	    TRAPD(err,
			id = self->iCategoryManager->GetCategoryL(categoryNameDesc);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return Py_BuildValue("i", id);
	}
	return 0;
}

static PyObject* categorymanager_GetGlobalCategory(obj_CategoryManager* self, PyObject* args)
{
	TPosLmGlobalCategory globalId;
	if (PyArg_ParseTuple(args, "i", &globalId)) {
		TPosLmItemId id;
	    TRAPD(err,
			id = self->iCategoryManager->GetGlobalCategoryL(globalId);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return Py_BuildValue("i", id);
	}
	return 0;
}

static PyObject* categorymanager_GlobalCategoryName(obj_CategoryManager* self, PyObject* args)
{
	TPosLmGlobalCategory globalId;
	if (PyArg_ParseTuple(args, "i", &globalId)) {
		HBufC16* name;
	    TRAPD(err,
			name = self->iCategoryManager->GlobalCategoryNameL(globalId);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    if (name == 0) {
	    	RETURN_PYNONE
	    }
	    return Py_BuildValue("u#", name->Ptr(), name->Length());
	}
	return 0;
}

static PyObject* categorymanager_ResetGlobalCategories(obj_CategoryManager* self, PyObject* /*args*/)
{
	CPosLmOperation* operation;
    TRAPD(err,
		operation = self->iCategoryManager->ResetGlobalCategoriesL();
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)createOperation((PyObject*)self, operation);
}

static PyObject* categorymanager_AddCategoryToLandmarks(obj_CategoryManager* self, PyObject* args)
{
	TPosLmItemId id;
	PyObject* idList;
	if (PyArg_ParseTuple(args, "iO", &id, &idList)) {
	    if (!PyList_Check(idList))
		{
	        PyErr_SetString(PyExc_TypeError, "Second argument must be a list");
		    return 0;
		}

		bool error = false;
		RArray<TPosLmItemId> idArray;

		copyItemIdArray(idList, idArray, error);

	    if (error)
	    {
	    	idArray.Close();
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

		CPosLmOperation* operation;
	    TRAPD(err,
			operation = self->iCategoryManager->AddCategoryToLandmarksL(id, idArray);
	    );

		idArray.Close();

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static PyObject* categorymanager_RemoveCategoryFromLandmarks(obj_CategoryManager* self, PyObject* args)
{
	TPosLmItemId id;
	PyObject* idList;
	if (PyArg_ParseTuple(args, "iO", &id, &idList)) {
	    if (!PyList_Check(idList))
		{
	        PyErr_SetString(PyExc_TypeError, "Second argument must be a list");
		    return 0;
		}

		bool error = false;
		RArray<TPosLmItemId> idArray;

		copyItemIdArray(idList, idArray, error);

	    if (error)
	    {
	    	idArray.Close();
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

		CPosLmOperation* operation;
	    TRAPD(err,
			operation = self->iCategoryManager->RemoveCategoryFromLandmarksL(id, idArray);
	    );

		idArray.Close();

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static PyObject* categorymanager_RemoveCategories(obj_CategoryManager* self, PyObject* args)
{
	PyObject* idList;
	if (PyArg_ParseTuple(args, "O", &idList)) {
	    if (!PyList_Check(idList))
		{
	        PyErr_SetString(PyExc_TypeError, "Argument must be a list");
		    return 0;
		}

		bool error = false;
		RArray<TPosLmItemId> idArray;

		copyItemIdArray(idList, idArray, error);

	    if (error)
	    {
	    	idArray.Close();
	        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
		    return 0;
	    }

		CPosLmOperation* operation;
	    TRAPD(err,
			operation = self->iCategoryManager->RemoveCategoriesL(idArray);
	    );

		idArray.Close();

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static const PyMethodDef CategoryManager_methods[] =
{
    {"Close", (PyCFunction)categorymanager_Close, METH_NOARGS, "void Close()"},
    {"CategoryIterator", (PyCFunction)categorymanager_CategoryIterator, METH_VARARGS, "ItemIterator CategoryIterator()"},
    {"ReferencedCategoryIterator", (PyCFunction)categorymanager_ReferencedCategoryIterator, METH_VARARGS, "ItemIterator ReferencedCategoryIterator()"},
    {"ReadCategory", (PyCFunction)categorymanager_ReadCategory, METH_VARARGS, "LandmarkCategory ReadCategory()"},
    {"AddCategory", (PyCFunction)categorymanager_AddCategory, METH_VARARGS, "int AddLandmark(LandmarkCategory category)" },
    {"UpdateCategory", (PyCFunction)categorymanager_UpdateCategory, METH_VARARGS, "void UpdateCategory(LandmarkCategory category)" },
    {"RemoveCategory", (PyCFunction)categorymanager_RemoveCategory, METH_VARARGS, "void RemoveCategory(int categoryId)" },
    {"GetCategory", (PyCFunction)categorymanager_GetCategory, METH_VARARGS, "int GetCategory(string name)" },
    {"GetGlobalCategory", (PyCFunction)categorymanager_GetGlobalCategory, METH_VARARGS, "int GetGlobalCategory(int globalId)" },
    {"GlobalCategoryName", (PyCFunction)categorymanager_GlobalCategoryName, METH_VARARGS, "string GlobalCategoryName(int globalId)" },
    {"ResetGlobalCategories", (PyCFunction)categorymanager_ResetGlobalCategories, METH_NOARGS, "LmOperation ResetGlobalCategories()" },
    {"AddCategoryToLandmarks", (PyCFunction)categorymanager_AddCategoryToLandmarks, METH_VARARGS, "LmOperation AddCategoryToLandmarks(int categoryId, [int] landmarkIds)" },
    {"RemoveCategoryFromLandmarks", (PyCFunction)categorymanager_RemoveCategoryFromLandmarks, METH_VARARGS, "LmOperation RemoveCategoryFromLandmarks(int categoryId, [int] landmarkIds)" },
    {"RemoveCategories", (PyCFunction)categorymanager_RemoveCategories, METH_VARARGS, "LmOperation RemoveCategories([int] categoryIds)" },
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_CategoryManager(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&CategoryManager_methods[0]), self, name);
}

static const PyTypeObject type_template_CategoryManager =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.CategoryManager",            /*tp_name*/
    sizeof(obj_CategoryManager), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_CategoryManager, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_CategoryManager, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// LandmarkCategory ///////////////////////////////////////////////////////////////////////////////////////////

// ctor (factory-function)
static PyObject* landmarkcategory_CreateLandmarkCategory(PyObject* /*self*/, PyObject* /*args*/)
{
    obj_LandmarkCategory *category = PyObject_New(obj_LandmarkCategory, &type_LandmarkCategory);

    if (!category)
    {
        PyErr_SetString(PyExc_TypeError, "Alloc error");
        return 0;
    }

    TRAPD(err,
        category->iLandmarkCategory = CPosLandmarkCategory::NewL();
    );

    if (err)
    {
        PyObject_Del(category);
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)category;
}

// dtor
static void dealloc_LandmarkCategory(obj_LandmarkCategory* self)
{
	if (self->iLandmarkCategory) delete self->iLandmarkCategory;
	self->iLandmarkCategory = NULL;
    PyObject_Del(self);
}

static PyObject* landmarkcategory_Close(obj_LandmarkCategory* self, PyObject* /*args*/)
{
	if (self->iLandmarkCategory) delete self->iLandmarkCategory;
	self->iLandmarkCategory = 0;
	RETURN_PYNONE
}

static PyObject* landmarkcategory_SetIcon(obj_LandmarkCategory* self, PyObject* args)
{
	char* name;
	int nameLen;
	int iconIndex;
	int iconMaskIndex;
	if (PyArg_ParseTuple(args, "u#ii", &name, &nameLen, &iconIndex, &iconMaskIndex)) {
        TPtrC nameDesc((TUint16*)name, nameLen);
		TRAPD(err,
			self->iLandmarkCategory->SetIconL(nameDesc, iconIndex, iconMaskIndex);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkcategory_GetIcon(obj_LandmarkCategory* self, PyObject* /*args*/)
{
	int iconIndex;
	int iconMaskIndex;
    TPtrC nameDesc;
	int err = self->iLandmarkCategory->GetIcon(nameDesc, iconIndex, iconMaskIndex);
	if (err) {
		// don't throw exception, just return None
		RETURN_PYNONE
	}
	return Py_BuildValue("u#ii", nameDesc.Ptr(), nameDesc.Length(), iconIndex, iconMaskIndex);
}

static PyObject* landmarkcategory_GetCategoryName(obj_LandmarkCategory* self, PyObject* /*args*/)
{
	TPtrC name;
	int err = self->iLandmarkCategory->GetCategoryName(name);
	if (err) {
		// don't throw exception, just return None
		RETURN_PYNONE
	}
	return Py_BuildValue("u#", name.Ptr(), name.Length());
}

static PyObject* landmarkcategory_SetCategoryName(obj_LandmarkCategory* self, PyObject* args)
{
	char* name;
	int nameLen;
	if (PyArg_ParseTuple(args, "u#", &name, &nameLen)) {
        TPtrC nameDesc((TUint16*)name, nameLen);
		TRAPD(err,
			self->iLandmarkCategory->SetCategoryNameL(nameDesc);
		);
	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkcategory_GlobalCategory(obj_LandmarkCategory* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmarkCategory->GlobalCategory());
}

static PyObject* landmarkcategory_CategoryId(obj_LandmarkCategory* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmarkCategory->CategoryId());
}

static PyObject* landmarkcategory_RemoveCategoryAttributes(obj_LandmarkCategory* self, PyObject* args)
{
	CPosLandmarkCategory::TAttributes attr;
	if (PyArg_ParseTuple(args, "i", &attr)) {
		self->iLandmarkCategory->RemoveCategoryAttributes(attr);
		RETURN_PYNONE
	}
	return 0;
}

static const PyMethodDef LandmarkCategory_methods[] =
{
    {"Close", (PyCFunction)landmarkcategory_Close, METH_NOARGS, "void Close()"},
    {"GetIcon", (PyCFunction)landmarkcategory_GetIcon, METH_NOARGS, "(iconFileName, iconIndex, iconMaskIndex) GetIcon()"},
    {"SetIcon", (PyCFunction)landmarkcategory_SetIcon, METH_VARARGS, "void SetIcon(iconFileName, iconIndex, iconMaskIndex)"},
    {"GetCategoryName", (PyCFunction)landmarkcategory_GetCategoryName, METH_NOARGS, "string GetCategoryName()"},
    {"SetCategoryName", (PyCFunction)landmarkcategory_SetCategoryName, METH_VARARGS, "void SetCategoryName(string name)"},
    {"CategoryId", (PyCFunction)landmarkcategory_CategoryId, METH_NOARGS, "int CategoryId()"},
    {"GlobalCategory", (PyCFunction)landmarkcategory_GlobalCategory, METH_NOARGS, "int GlobalCategory()"},
    {"RemoveCategoryAttributes", (PyCFunction)landmarkcategory_RemoveCategoryAttributes, METH_NOARGS, "void RemoveCategoryAttributes()"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_LandmarkCategory(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&LandmarkCategory_methods[0]), self, name);
}

static const PyTypeObject type_template_LandmarkCategory =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.LandmarkCategory",            /*tp_name*/
    sizeof(obj_LandmarkCategory), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_LandmarkCategory, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_LandmarkCategory, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// DatabaseManager //////////////////////////////////////////////////////////////////////////////////////////////

static PyObject* databasemanager_CreateDatabaseManager(PyObject* /*self*/, PyObject* /*args*/)
{
	CPosLmDatabaseManager* manager;

    TRAPD(err,
        manager = CPosLmDatabaseManager::NewL();
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

    obj_DatabaseManager *databaseManager = PyObject_New(obj_DatabaseManager, &type_DatabaseManager);

    if (databaseManager)
    {
        databaseManager->iDatabaseManager = manager;
    }

    return (PyObject*)databaseManager;
}

// dtor
static void dealloc_DatabaseManager(obj_DatabaseManager* databaseManager)
{
	if (databaseManager->iDatabaseManager) delete databaseManager->iDatabaseManager;
	databaseManager->iDatabaseManager = NULL;
    PyObject_Del(databaseManager);
}

static PyObject* databasemanager_Close(obj_DatabaseManager* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iDatabaseManager) delete self->iDatabaseManager;
		self->iDatabaseManager = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* databasemanager_DefaultDatabaseUri(obj_DatabaseManager* self, PyObject* /*args*/) {
	PyObject* result;
	TRAPD(err,
		HBufC* uri = self->iDatabaseManager->DefaultDatabaseUriLC();
		result = Py_BuildValue("u#", uri->Ptr(), uri->Length());
		CleanupStack::PopAndDestroy();
	);
    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }
    return result;
}

static PyObject* databasemanager_SetDefaultDatabaseUri(obj_DatabaseManager* self, PyObject* args) {
	char* uri;
	int uriLen;

	if (PyArg_ParseTuple(args, "u#:SetDefaultDatabaseUri", &uri, &uriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);
		TRAPD(err,
			self->iDatabaseManager->SetDefaultDatabaseUriL(uriDesc);
		);
		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_ListDatabases(obj_DatabaseManager* self, PyObject* args)
{
	char* protocol;
	int protocolLen;

	if (PyArg_ParseTuple(args, "u#:ListDatabases", &protocol, &protocolLen))
    {
        TPtrC* protocolDesc;
        if (protocolLen == 0)
        {
        	protocolDesc = new TPtrC(KNullDesC);
	    }
	    else
	    {
	    	protocolDesc = new TPtrC((TUint16*)protocol, protocolLen);
    	}
		PyObject* result;

		TRAPD(err,
			CDesCArray* descArray = self->iDatabaseManager->ListDatabasesLC(*protocolDesc);

			int count = descArray->Count();
			result = PyList_New(count);
			for (int i = 0; i < count; i++)
			{
				TPtrC item = (*descArray)[i];
				PyList_SetItem(result, i, Py_BuildValue("u#", item.Ptr(), item.Length())); // Note: SetItem steals a reference
			}
			CleanupStack::PopAndDestroy();
		);
		delete protocolDesc;

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    return result;
	}
	return 0;
}

static PyObject* databasemanager_ListDatabaseInfos(obj_DatabaseManager* self, PyObject* args)
{
	char* protocol;
	int protocolLen;

	if (PyArg_ParseTuple(args, "u#:ListDatabaseInfos", &protocol, &protocolLen))
    {
        TPtrC* protocolDesc;
        if (protocolLen == 0)
        {
        	protocolDesc = new TPtrC(KNullDesC);
	    }
	    else
	    {
	    	protocolDesc = new TPtrC((TUint16*)protocol, protocolLen);
    	}
		PyObject* result;

		TRAPD(err,
			RPointerArray< HPosLmDatabaseInfo > databaseInfoArray;
			self->iDatabaseManager->ListDatabasesL(databaseInfoArray, *protocolDesc);

			int count = databaseInfoArray.Count();
			result = PyList_New(count);
			for (int i = 0; i < count; i++)
			{
				HPosLmDatabaseInfo* item = databaseInfoArray[i];
				PyList_SetItem(result, i, (PyObject*)createDatabaseInfo((PyObject*)self, item)); // Note: SetItem steals a reference
			}
			databaseInfoArray.Close();
		);
		delete protocolDesc;

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    return result;
	}
	return 0;
}

static PyObject* databasemanager_RegisterDatabase(obj_DatabaseManager* self, PyObject* args)
{
	obj_DatabaseInfo* info;
	if (PyArg_ParseTuple(args, "O!:RegisterDatabase", &type_DatabaseInfo, &info))
    {
		TRAPD(err,
			self->iDatabaseManager->RegisterDatabaseL(*(info->iDatabaseInfo));
		);

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_UnregisterDatabase(obj_DatabaseManager* self, PyObject* args) {
	char* uri;
	int uriLen;

	if (PyArg_ParseTuple(args, "u#:UnregisterDatabase", &uri, &uriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);
		TRAPD(err,
			self->iDatabaseManager->UnregisterDatabaseL(uriDesc);
		);
		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_UnregisterAllDatabases(obj_DatabaseManager* self, PyObject* args) {
	char* protocol;
	int protocolLen;

	if (PyArg_ParseTuple(args, "u#:UnregisterAllDatabases", &protocol, &protocolLen))
    {
        TPtrC protocolDesc((TUint16*)protocol, protocolLen);
		TRAPD(err,
			self->iDatabaseManager->UnregisterAllDatabasesL(protocolDesc);
		);
		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_ModifyDatabaseSettings(obj_DatabaseManager* self, PyObject* args)
{
	obj_DatabaseSettings* settings;
	char* uri;
	int uriLen;
	if (PyArg_ParseTuple(args, "O!u#:ModifyDatabaseSettings", &type_DatabaseSettings, &settings, &uri, &uriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);
		TRAPD(err,
			self->iDatabaseManager->ModifyDatabaseSettingsL(uriDesc, *(settings->iDatabaseSettings));
		);

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_GetDatabaseInfo(obj_DatabaseManager* self, PyObject* args)
{
	obj_DatabaseInfo* info;
	if (PyArg_ParseTuple(args, "O!:GetDatabaseInfo", &type_DatabaseInfo, &info))
    {
		TRAPD(err,
			self->iDatabaseManager->GetDatabaseInfoL(*(info->iDatabaseInfo));
		);

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_CreateDatabase(obj_DatabaseManager* self, PyObject* args)
{
	obj_DatabaseInfo* info;
	if (PyArg_ParseTuple(args, "O!:CreateDatabase", &type_DatabaseInfo, &info))
    {
		TRAPD(err,
			self->iDatabaseManager->CreateDatabaseL(*(info->iDatabaseInfo));
		);

		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_DeleteDatabase(obj_DatabaseManager* self, PyObject* args) {
	char* uri;
	int uriLen;

	if (PyArg_ParseTuple(args, "u#:DeleteDatabase", &uri, &uriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);
		TRAPD(err,
			self->iDatabaseManager->DeleteDatabaseL(uriDesc);
		);
		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* databasemanager_DatabaseExists(obj_DatabaseManager* self, PyObject* args) {
	char* uri;
	int uriLen;

	if (PyArg_ParseTuple(args, "u#:DatabaseExists", &uri, &uriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);
        TBool result;
		TRAPD(err,
			result = self->iDatabaseManager->DatabaseExistsL(uriDesc);
		);
	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return Py_BuildValue("i", result);
	}
	return 0;
}

static PyObject* databasemanager_CopyDatabase(obj_DatabaseManager* self, PyObject* args) {
	char* uri;
	int uriLen;
	char* destUri;
	int destUriLen;

	if (PyArg_ParseTuple(args, "u#u#:CopyDatabase", &uri, &uriLen, &destUri, &destUriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);
        TPtrC destUriDesc((TUint16*)destUri, destUriLen);
		TRAPD(err,
			self->iDatabaseManager->CopyDatabaseL(uriDesc, destUriDesc);
		);
		RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static const PyMethodDef DatabaseManager_methods[] =
{
    {"Close", (PyCFunction)databasemanager_Close, METH_NOARGS, "void Close()"},
    {"DefaultDatabaseUri", (PyCFunction)databasemanager_DefaultDatabaseUri, METH_NOARGS, "string DefaultDatabaseUri()"},
    {"SetDefaultDatabaseUri", (PyCFunction)databasemanager_SetDefaultDatabaseUri, METH_VARARGS, "void SetDefaultDatabaseUri(string uri)"},
    {"ListDatabases", (PyCFunction)databasemanager_ListDatabases, METH_VARARGS, "[string] ListDatabases(string protocol)"},
    {"ListDatabaseInfos", (PyCFunction)databasemanager_ListDatabaseInfos, METH_VARARGS, "[DatabaseInfo] ListDatabaseInfos(string protocol)"},
    {"RegisterDatabase", (PyCFunction)databasemanager_RegisterDatabase, METH_VARARGS, "void RegisterDatabase(DatabaseInfo info)"},
    {"UnregisterDatabase", (PyCFunction)databasemanager_UnregisterDatabase, METH_VARARGS, "void UnregisterDatabase(string uri)"},
    {"UnregisterAllDatabases", (PyCFunction)databasemanager_UnregisterAllDatabases, METH_VARARGS, "void UnregisterAllDatabases(string protocol)"},
    {"ModifyDatabaseSettings", (PyCFunction)databasemanager_ModifyDatabaseSettings, METH_VARARGS, "void ModifyDatabaseSettings(string uri, DatabaseSettings settings)"},
    {"GetDatabaseInfo", (PyCFunction)databasemanager_GetDatabaseInfo, METH_VARARGS, "void GetDatabaseInfo(DatabaseInfo info)"},
    {"CreateDatabase", (PyCFunction)databasemanager_CreateDatabase, METH_VARARGS, "void CreateDatabase(DatabaseInfo info)"},
    {"DeleteDatabase", (PyCFunction)databasemanager_DeleteDatabase, METH_VARARGS, "void DeleteDatabase(string uri)"},
    {"DatabaseExists", (PyCFunction)databasemanager_DatabaseExists, METH_VARARGS, "boolean DatabaseExists(string uri)"},
    {"CopyDatabase", (PyCFunction)databasemanager_CopyDatabase, METH_VARARGS, "void CopyDatabase(string srcUri, string destUri)"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_DatabaseManager(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&DatabaseManager_methods[0]), self, name);
}

static const PyTypeObject type_template_DatabaseManager =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.DatabaseManager",            /*tp_name*/
    sizeof(obj_DatabaseManager), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_DatabaseManager, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_DatabaseManager, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// DatabaseInfo //////////////////////////////////////////////////////////////////////////////////////////////

static PyObject* databaseinfo_CreateDatabaseInfo(PyObject* /*self*/, PyObject* args)
{
	char* uri;
	int uriLen;

	if (PyArg_ParseTuple(args, "u#:CreateDatabaseInfo", &uri, &uriLen))
    {
        TPtrC uriDesc((TUint16*)uri, uriLen);

		HPosLmDatabaseInfo* info;

	    TRAPD(err,
	        info = HPosLmDatabaseInfo::NewL(uriDesc);
	    );

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

	    return (PyObject*)createDatabaseInfo((PyObject*)NULL, info);
	}
	return 0;
}

// dtor
static void dealloc_DatabaseInfo(obj_DatabaseInfo* databaseInfo)
{
	// NOTE: DON'T delete a H* thingy (learned that the hard way ...)
//	if (databaseInfo->iDatabaseInfo) delete databaseInfo->iDatabaseInfo;
	if (databaseInfo->iParent) Py_DECREF(databaseInfo->iParent);
	databaseInfo->iParent = NULL;
	PyObject_Del(databaseInfo);
}

static PyObject* databaseinfo_Close(obj_DatabaseInfo* self, PyObject* /*args*/) {
	TRAPD(err,
//		if (self->iDatabaseInfo) delete self->iDatabaseInfo;
		self->iDatabaseInfo = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* databaseinfo_DatabaseUri(obj_DatabaseInfo* self, PyObject* /*args*/) {
	TPtrC uri = self->iDatabaseInfo->DatabaseUri();
	return Py_BuildValue("u#", uri.Ptr(), uri.Length());
}

static PyObject* databaseinfo_Protocol(obj_DatabaseInfo* self, PyObject* /*args*/) {
	TPtrC protocol = self->iDatabaseInfo->Protocol();
	return Py_BuildValue("u#", protocol.Ptr(), protocol.Length());
}

static PyObject* databaseinfo_IsDefault(obj_DatabaseInfo* self, PyObject* /*args*/) {
	return Py_BuildValue("i", self->iDatabaseInfo->IsDefault());
}

static PyObject* databaseinfo_DatabaseMedia (obj_DatabaseInfo* self, PyObject* /*args*/) {
	return Py_BuildValue("i", self->iDatabaseInfo->DatabaseMedia());
}

static PyObject* databaseinfo_DatabaseDrive (obj_DatabaseInfo* self, PyObject* /*args*/) {
	return Py_BuildValue("c", (int)self->iDatabaseInfo->DatabaseDrive());
}

static PyObject* databaseinfo_Settings (obj_DatabaseInfo* self, PyObject* /*args*/) {
	return (PyObject*)createDatabaseSettings((PyObject*)self, &(self->iDatabaseInfo->Settings()));
}

static PyObject* databaseinfo_Size(obj_DatabaseInfo* self, PyObject* /*args*/) {
	return Py_BuildValue("i", self->iDatabaseInfo->Size());
}

static const PyMethodDef DatabaseInfo_methods[] =
{
    {"Close", (PyCFunction)databaseinfo_Close, METH_NOARGS, "void Close()"},
    {"DatabaseUri", (PyCFunction)databaseinfo_DatabaseUri, METH_NOARGS, "string DatabaseUri()"},
    {"Protocol", (PyCFunction)databaseinfo_Protocol, METH_NOARGS, "string Protocol()"},
    {"IsDefault", (PyCFunction)databaseinfo_IsDefault, METH_NOARGS, "boolean IsDefault()"},
    {"DatabaseMedia", (PyCFunction)databaseinfo_DatabaseMedia, METH_NOARGS, "int DatabaseMedia()"},
    {"DatabaseDrive", (PyCFunction)databaseinfo_DatabaseDrive, METH_NOARGS, "string DatabaseDrive()"},
    {"Settings", (PyCFunction)databaseinfo_Settings, METH_NOARGS, "DatabaseSettings Settings()"},
    {"Size", (PyCFunction)databaseinfo_Size, METH_NOARGS, "int Size()"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_DatabaseInfo(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&DatabaseInfo_methods[0]), self, name);
}

static const PyTypeObject type_template_DatabaseInfo =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.DatabaseInfo",            /*tp_name*/
    sizeof(obj_DatabaseInfo), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_DatabaseInfo, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_DatabaseInfo, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// DatabaseSettings //////////////////////////////////////////////////////////////////////////////////////////////

// dtor
static void dealloc_DatabaseSettings(obj_DatabaseSettings* databaseSettings)
{
	if (databaseSettings->iDatabaseSettings) delete databaseSettings->iDatabaseSettings;
	databaseSettings->iDatabaseSettings = NULL;
	if (databaseSettings->iParent) Py_DECREF(databaseSettings->iParent);
	databaseSettings->iParent = NULL;
	PyObject_Del(databaseSettings);
}

static PyObject* databasesettings_Close(obj_DatabaseSettings* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iDatabaseSettings) delete self->iDatabaseSettings;
		self->iDatabaseSettings = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* databasesettings_IsAttributeSet(obj_DatabaseSettings* self, PyObject* args) {
	TPosLmDatabaseSettings::TAttribute attribute;

	if (PyArg_ParseTuple(args, "i", &attribute))
    {
		return Py_BuildValue("i", self->iDatabaseSettings->IsAttributeSet(attribute));
	}
    return 0;
}

static PyObject* databasesettings_UnsetAttribute(obj_DatabaseSettings* self, PyObject* args) {
	TPosLmDatabaseSettings::TAttribute attribute;

	if (PyArg_ParseTuple(args, "i", &attribute))
    {
		self->iDatabaseSettings->UnsetAttribute(attribute);
		RETURN_PYNONE
	}
    return 0;
}

static PyObject* databasesettings_DatabaseName(obj_DatabaseSettings* self, PyObject* /*args*/) {
	TPtrC name = self->iDatabaseSettings->DatabaseName();
	return Py_BuildValue("u#", name.Ptr(), name.Length());
}

static PyObject* databasesettings_SetDatabaseName(obj_DatabaseSettings* self, PyObject* args) {
	char* name;
	int nameLen;

	if (PyArg_ParseTuple(args, "u#:SetDatabaseName", &name, &nameLen))
    {
        TPtrC nameDesc((TUint16*)name, nameLen);
		self->iDatabaseSettings->SetDatabaseName(nameDesc);
    }
    return 0;
}


static const PyMethodDef DatabaseSettings_methods[] =
{
    {"Close", (PyCFunction)databasesettings_Close, METH_NOARGS, "void Close()"},
    {"IsAttributeSet", (PyCFunction)databasesettings_IsAttributeSet, METH_VARARGS, "int IsAttributeSet(int attribute)"},
    {"UnsetAttribute", (PyCFunction)databasesettings_UnsetAttribute, METH_VARARGS, "void UnsetAttribute(int attribute)"},
    {"DatabaseName", (PyCFunction)databasesettings_DatabaseName, METH_NOARGS, "string DatabaseName()"},
    {"SetDatabaseName", (PyCFunction)databasesettings_SetDatabaseName, METH_VARARGS, "void SetDatabaseName(string name)"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_DatabaseSettings(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&DatabaseSettings_methods[0]), self, name);
}

static const PyTypeObject type_template_DatabaseSettings =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.DatabaseSettings",            /*tp_name*/
    sizeof(obj_DatabaseSettings), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_DatabaseSettings, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_DatabaseSettings, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// LandmarkEncoder //////////////////////////////////////////////////////////////////////////////////////////////

static PyObject* landmarkencoder_CreateLandmarkEncoder(PyObject* /*self*/, PyObject* args)
{
	char* mime;
	int mimeLen;

	if (PyArg_ParseTuple(args, "s#:CreateLandmarkEncoder", &mime, &mimeLen))
    {
        TPtr8 mimeDesc((TUint8*)mime, mimeLen);
		CPosLandmarkEncoder* encoder;
	
	    TRAPD(err,
	        encoder = CPosLandmarkEncoder::NewL(mimeDesc);
	    );
	
	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    obj_LandmarkEncoder *landmarkEncoder = PyObject_New(obj_LandmarkEncoder, &type_LandmarkEncoder);
	
	    if (landmarkEncoder)
	    {
	        landmarkEncoder->iLandmarkEncoder = encoder;
	    }
	
	    return (PyObject*)landmarkEncoder;
	}
	return 0;
}

// dtor
static void dealloc_LandmarkEncoder(obj_LandmarkEncoder* landmarkEncoder)
{
	if (landmarkEncoder->iLandmarkEncoder) delete landmarkEncoder->iLandmarkEncoder;
	landmarkEncoder->iLandmarkEncoder = NULL;
	PyObject_Del(landmarkEncoder);
}

static PyObject* landmarkencoder_Close(obj_LandmarkEncoder* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iLandmarkEncoder) delete self->iLandmarkEncoder;
		self->iLandmarkEncoder = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* landmarkencoder_AddCategoryForLatestLandmark(obj_LandmarkEncoder* self, PyObject* args)
{
	obj_LandmarkCategory* category;
	if (PyArg_ParseTuple(args, "O!:AddCategoryForLatestLandmark", &type_LandmarkCategory, &category)) 
	{
	    TRAPD(err,
			self->iLandmarkEncoder->AddCategoryForLatestLandmarkL(*(category->iLandmarkCategory));
	    );

	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkencoder_AddLandmark(obj_LandmarkEncoder* self, PyObject* args)
{
	obj_Landmark* landmark;
	if (PyArg_ParseTuple(args, "O!:AddLandmark", &type_Landmark, &landmark)) 
	{
	    TRAPD(err,
			self->iLandmarkEncoder->AddLandmarkL(*(landmark->iLandmark));
	    );

	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkencoder_AddCollectionData(obj_LandmarkEncoder* self, PyObject* args)
{
	TPosLmCollectionDataId dataId;
	char* data;
	int dataLen;
	if (PyArg_ParseTuple(args, "iu#:AddCollectionData", &dataId, &data, &dataLen)) 
	{
        TPtrC dataDesc((TUint16*)data, dataLen);
	    TRAPD(err,
			self->iLandmarkEncoder->AddCollectionDataL(dataId, dataDesc);
	    );

	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkencoder_FinalizeEncoding(obj_LandmarkEncoder* self, PyObject* /*args*/)
{
	CPosLmOperation* operation;
    TRAPD(err,
		operation = self->iLandmarkEncoder->FinalizeEncodingL();
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return (PyObject*)createOperation((PyObject*)self, operation);
}

static PyObject* landmarkencoder_SetOutputFile(obj_LandmarkEncoder* self, PyObject* args)
{
	char* name;
	int nameLen;
	if (PyArg_ParseTuple(args, "u#:SetOutputFile", &name, &nameLen)) 
	{
        TPtrC nameDesc((TUint16*)name, nameLen);
	    TRAPD(err,
			self->iLandmarkEncoder->SetOutputFileL(nameDesc);
	    );

	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkencoder_SetUseOutputBuffer(obj_LandmarkEncoder* self, PyObject* /*args*/)
{
	CBufBase* bufBase;
    TRAPD(err,
		bufBase = self->iLandmarkEncoder->SetUseOutputBufferL();
    );

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

    obj_CBufBase *pyBufBase = PyObject_New(obj_CBufBase, &type_CBufBase);

    if (pyBufBase)
    {
        pyBufBase->iBufBase = bufBase;
    }

    return (PyObject*)pyBufBase;
}

static const PyMethodDef LandmarkEncoder_methods[] =
{
    {"Close", (PyCFunction)landmarkencoder_Close, METH_NOARGS, "void Close()"},
    {"AddCategoryForLatestLandmark", (PyCFunction)landmarkencoder_AddCategoryForLatestLandmark, METH_NOARGS, "void AddCategoryForLatestLandmark(LandmarkCategory category)"},
    {"AddLandmark", (PyCFunction)landmarkencoder_AddLandmark, METH_NOARGS, "void AddLandmark(Landmark landmark)"},
    {"AddCollectionData", (PyCFunction)landmarkencoder_AddCollectionData, METH_NOARGS, "void AddCollectionData(int dataId, string data)"},
    {"FinalizeEncoding", (PyCFunction)landmarkencoder_FinalizeEncoding, METH_NOARGS, "void FinalizeEncoding()"},
    {"SetOutputFile", (PyCFunction)landmarkencoder_SetOutputFile, METH_NOARGS, "void SetOutputFile(string fileName)"},
    {"SetUseOutputBuffer", (PyCFunction)landmarkencoder_SetUseOutputBuffer, METH_NOARGS, "CBufBase SetUseOutputBuffer()"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_LandmarkEncoder(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&LandmarkEncoder_methods[0]), self, name);
}

static const PyTypeObject type_template_LandmarkEncoder =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.LandmarkEncoder",            /*tp_name*/
    sizeof(obj_LandmarkEncoder), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_LandmarkEncoder, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_LandmarkEncoder, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// CBufBase //////////////////////////////////////////////////////////////////////////////////////////////

// dtor
static void dealloc_CBufBase(obj_CBufBase* cbufbase)
{
	if (cbufbase->iBufBase) delete cbufbase->iBufBase;
	cbufbase->iBufBase = NULL;
	PyObject_Del(cbufbase);
}

static PyObject* cbufbase_Close(obj_CBufBase* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iBufBase) 
		{
			self->iBufBase->Reset();
			delete self->iBufBase;
		}
		self->iBufBase = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* cbufbase_Size(obj_CBufBase* self, PyObject* /*args*/) {
	return Py_BuildValue("i", self->iBufBase->Size());
}

static PyObject* cbufbase_Ptr(obj_CBufBase* self, PyObject* args) {
	int aPos;
	if (PyArg_ParseTuple(args, "i:Ptr", &aPos)) 
	{
		TPtr8 data = self->iBufBase->Ptr(aPos);
		return Py_BuildValue("s#", data.Ptr(), data.Length());
	}
	return 0;
}

static const PyMethodDef CBufBase_methods[] =
{
    {"Close", (PyCFunction)cbufbase_Close, METH_NOARGS, "void Close()"},
    {"Size", (PyCFunction)cbufbase_Size, METH_NOARGS, "int Size()"},
    {"Ptr", (PyCFunction)cbufbase_Ptr, METH_NOARGS, "string Ptr(int index)"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_CBufBase(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&CBufBase_methods[0]), self, name);
}

static const PyTypeObject type_template_CBufBase =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.CBufBase",            /*tp_name*/
    sizeof(obj_CBufBase), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_CBufBase, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_CBufBase, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// LandmarkParser //////////////////////////////////////////////////////////////////////////////////////////////

static PyObject* landmarkparser_CreateLandmarkParser(PyObject* /*self*/, PyObject* args)
{
	char* mime;
	int mimeLen;

	if (PyArg_ParseTuple(args, "s#:CreateLandmarkParser", &mime, &mimeLen))
    {
        TPtr8 mimeDesc((TUint8*)mime, mimeLen);
		CPosLandmarkParser* parser;
	
	    TRAPD(err,
	        parser = CPosLandmarkParser::NewL(mimeDesc);
	    );
	
	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    obj_LandmarkParser *landmarkParser = PyObject_New(obj_LandmarkParser, &type_LandmarkParser);
	
	    if (landmarkParser)
	    {
	        landmarkParser->iLandmarkParser = parser;
	    }
	
	    return (PyObject*)landmarkParser;
	}
	return 0;
}

// dtor
static void dealloc_LandmarkParser(obj_LandmarkParser* landmarkParser)
{
	if (landmarkParser->iLandmarkParser) delete landmarkParser->iLandmarkParser;
	landmarkParser->iLandmarkParser = NULL;
	PyObject_Del(landmarkParser);
}

static PyObject* landmarkparser_Close(obj_LandmarkParser* self, PyObject* /*args*/) {
	TRAPD(err,
		if (self->iLandmarkParser) delete self->iLandmarkParser;
		self->iLandmarkParser = 0;
	);
    RETURN_ERROR_OR_PYNONE(err);
}

static PyObject* landmarkparser_SetInputFile(obj_LandmarkParser* self, PyObject* args)
{
	char* name;
	int nameLen;
	if (PyArg_ParseTuple(args, "u#:SetInputFile", &name, &nameLen)) 
	{
        TPtrC nameDesc((TUint16*)name, nameLen);
	    TRAPD(err,
			self->iLandmarkParser->SetInputFileL(nameDesc);
	    );

	    RETURN_ERROR_OR_PYNONE(err);
	}
	return 0;
}

static PyObject* landmarkparser_SetInputBuffer(obj_LandmarkParser* self, PyObject* args)
{
	char* buf;
	int bufLen;
	if (PyArg_ParseTuple(args, "s#:SetInputBuffer", &buf, &bufLen)) 
	{
        TPtr8 bufDesc((TUint8*)buf, bufLen);
		self->iLandmarkParser->SetInputBuffer(bufDesc);

	    RETURN_PYNONE
	}
	return 0;
}

static PyObject* landmarkparser_ParseContent(obj_LandmarkParser* self, PyObject* args)
{
	int buildIndex;
	if (PyArg_ParseTuple(args, "i:ParseContent", &buildIndex)) 
	{
		CPosLmOperation* operation;
	    TRAPD(err,
    		operation = self->iLandmarkParser->ParseContentL(buildIndex != 0);
    	);

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    return (PyObject*)createOperation((PyObject*)self, operation);
	}
	return 0;
}

static PyObject* landmarkparser_NumOfParsedLandmarks(obj_LandmarkParser* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmarkParser->NumOfParsedLandmarks());
}

static PyObject* landmarkparser_FirstCollectionDataId(obj_LandmarkParser* self, PyObject* /*args*/)
{
	return Py_BuildValue("i", self->iLandmarkParser->FirstCollectionDataId());
}

static PyObject* landmarkparser_NextCollectionDataId(obj_LandmarkParser* self, PyObject* args)
{
	TPosLmCollectionDataId prevId;
	if (PyArg_ParseTuple(args, "i:NextCollectionDataId", &prevId)) 
	{
		return Py_BuildValue("i", self->iLandmarkParser->NextCollectionDataId(prevId));
	}
	return 0;
}

static PyObject* landmarkparser_CollectionData(obj_LandmarkParser* self, PyObject* args)
{
	TPosLmCollectionDataId id;
	if (PyArg_ParseTuple(args, "i:CollectionData", &id)) 
	{
		TPtrC data = self->iLandmarkParser->CollectionData(id);
		return Py_BuildValue("u#", data.Ptr(), data.Length());
	}
	return 0;
}

static PyObject* landmarkparser_Landmark(obj_LandmarkParser* self, PyObject* args)
{
	int index;
	if (PyArg_ParseTuple(args, "i:Landmark", &index)) 
	{
		CPosLandmark* parsedLandmark;
	    TRAPD(err,
    		parsedLandmark = self->iLandmarkParser->LandmarkLC(index);
    		CleanupStack::Pop();
    	);

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    obj_Landmark *landmark = PyObject_New(obj_Landmark, &type_Landmark);
	
	    if (!landmark)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }
        landmark->iLandmark = parsedLandmark;

	    return (PyObject*)landmark;
	}
	return 0;
}

static PyObject* landmarkparser_LandmarkCategory(obj_LandmarkParser* self, PyObject* args)
{
	int index;
	if (PyArg_ParseTuple(args, "i:LandmarkCategory", &index)) 
	{
		CPosLandmarkCategory* parsedLandmarkCategory;
	    TRAPD(err,
    		parsedLandmarkCategory = self->iLandmarkParser->LandmarkCategoryLC(index);
    		CleanupStack::Pop();
    	);

	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	
	    obj_LandmarkCategory *landmarkCategory = PyObject_New(obj_LandmarkCategory, &type_LandmarkCategory);
	
	    if (!landmarkCategory)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }
        landmarkCategory->iLandmarkCategory = parsedLandmarkCategory;

	    return (PyObject*)landmarkCategory;
	}
	return 0;
}

static const PyMethodDef LandmarkParser_methods[] =
{
    {"Close", (PyCFunction)landmarkparser_Close, METH_NOARGS, "void Close()"},
    {"SetInputFile", (PyCFunction)landmarkparser_SetInputFile, METH_VARARGS, "void SetInputFile(string name)"},
    {"SetInputBuffer", (PyCFunction)landmarkparser_SetInputBuffer, METH_VARARGS, "void SetInputBuffer(string buffer)"},
    {"ParseContent", (PyCFunction)landmarkparser_ParseContent, METH_VARARGS, "void ParseContent(bool buildIndex)"},
    {"NumOfParsedLandmarks", (PyCFunction)landmarkparser_NumOfParsedLandmarks, METH_NOARGS, "int NumOfParsedLandmarks()"},
    {"FirstCollectionDataId", (PyCFunction)landmarkparser_FirstCollectionDataId, METH_NOARGS, "int FirstCollectionDataId()"},
    {"NextCollectionDataId", (PyCFunction)landmarkparser_NextCollectionDataId, METH_VARARGS, "int NextCollectionDataId(int prevId)"},
    {"CollectionData", (PyCFunction)landmarkparser_CollectionData, METH_VARARGS, "string CollectionData(int id)"},
    {"Landmark", (PyCFunction)landmarkparser_Landmark, METH_VARARGS, "Landmark Landmark(int index)"},
    {"LandmarkCategory", (PyCFunction)landmarkparser_LandmarkCategory, METH_VARARGS, "LandmarkCategory LandmarkCategory(int id)"},
    {NULL, NULL} /* sentinel */
};

static PyObject *getattr_LandmarkParser(PyObject *self, char *name)
{
    return Py_FindMethod(const_cast<PyMethodDef*>(&LandmarkParser_methods[0]), self, name);
}

static const PyTypeObject type_template_LandmarkParser =
{
/*******************************************************/
    PyObject_HEAD_INIT(0)    /* initialize to 0 to ensure Win32 portability */
    0,                 /*ob_size*/
    "landmarks.LandmarkParser",            /*tp_name*/
    sizeof(obj_LandmarkParser), /*tp_basicsize*/
    0,                 /*tp_itemsize*/
    /* methods */
    (destructor)dealloc_LandmarkParser, /*tp_dealloc*/
	 0, /*tp_print*/
    (getattrfunc)getattr_LandmarkParser, /*tp_getattr*/

    /* implied by ISO C: all zeros thereafter */
};

/// Dialogs ///////////////////////////////////////////////////////////////////////////////////////
static PyObject* landmarks_ShowSelectDialog(PyObject* /*self*/, PyObject* args, int dialogType)
{
	int showEmptyCategories;
	TPosLmItemId selectedItem;
	PyObject* result;

	if (dialogType == 0)
	{
		if (!PyArg_ParseTuple(args, "ii:ShowSelectCategoryDialog", &showEmptyCategories, &selectedItem)) return 0;
	}
	else
	{
		if (!PyArg_ParseTuple(args, "i:ShowSelectLandmarkDialog", &selectedItem)) return 0;
	}
    int err;

    Py_BEGIN_ALLOW_THREADS

    TRAP(err, 
	    TLmkItemIdDbCombiInfo selectedDialogItem;
	    if (selectedItem != KPosLmNullItemId) selectedDialogItem.SetItemId(selectedItem);

	    int dialogResult;
	    if (dialogType == 0)
	    {
	      	CLmkCategorySelectorDlg* dlg = CLmkCategorySelectorDlg::NewL(showEmptyCategories != 0);
			dialogResult = dlg->ExecuteLD(selectedDialogItem);
		}
		else
		{
	      	CLmkLandmarkSelectorDlg* dlg = CLmkLandmarkSelectorDlg::NewL();
			dialogResult = dlg->ExecuteLD(selectedDialogItem);
		}
	    if (dialogResult == 0)
	    {
	    	Py_INCREF(Py_None);
	    	result = Py_BuildValue("(iiO)", 0, 0, Py_None);
	    }
	    else
	    {
    		result = Py_BuildValue("(iiO)", 1, selectedDialogItem.GetItemId(), createDatabase(selectedDialogItem.GetLmDb()));
    	}
    );

    Py_END_ALLOW_THREADS

    if (err)
    {
        return SPyErr_SetFromSymbianOSErr(err);
    }

    return result;
}

static PyObject* landmarks_ShowSelectMultipleDialog(PyObject* /*self*/, PyObject* args, int dialogType)
{
	int showEmptyCategories;
	PyObject* selectedItemList;

	if (dialogType == 0)
	{
		if (!PyArg_ParseTuple(args, "iO:ShowSelectMultipleCategoryDialog", &showEmptyCategories, &selectedItemList)) return 0;
	}
	else
	{
		if (!PyArg_ParseTuple(args, "iO:ShowSelectMultipleLandmarkDialog", &selectedItemList)) return 0;
	}
    if (!PyList_Check(selectedItemList))
	{
        PyErr_SetString(PyExc_TypeError, "Second argument must be a list");
	    return 0;
	}

	RArray<TLmkItemIdDbCombiInfo> selectedItemArray;

	int count = PyList_Size(selectedItemList);
	bool error = false;
	for (int i = 0; i < count; i++) 
	{
		PyObject* element = PyList_GetItem(selectedItemList, i);
		if (PyInt_Check(element)) 
		{
		    TLmkItemIdDbCombiInfo selectedDialogItem;
		    TPosLmItemId itemId = (TPosLmItemId)PyInt_AsLong(element);
		    selectedDialogItem.SetItemId(itemId);
			selectedItemArray.Append(selectedDialogItem);
		}
		else error = true;
		Py_DECREF(element);
	}

    if (error)
    {
    	selectedItemArray.Close();
        PyErr_SetString(PyExc_TypeError, "Not all list elements are ints");
	    return 0;
    }

    int dialogResult;
    int err;
    Py_BEGIN_ALLOW_THREADS
    TRAP(err, {
	    if (dialogType == 0)
	    {
	      	CLmkCategorySelectorDlg* dlg = CLmkCategorySelectorDlg::NewL(showEmptyCategories != 0);
			dialogResult = dlg->ExecuteLD(selectedItemArray);
		}
		else
		{
	      	CLmkLandmarkSelectorDlg* dlg = CLmkLandmarkSelectorDlg::NewL();
			dialogResult = dlg->ExecuteLD(selectedItemArray);
		}
    });
    Py_END_ALLOW_THREADS

    if (err)
    {
    	selectedItemArray.Close();
        return SPyErr_SetFromSymbianOSErr(err);
    }

    if (dialogResult == 0) {
    	selectedItemArray.Close();
    	Py_INCREF(Py_None);
    	return Py_BuildValue("(iO)", 0, Py_None);
    }

	count = selectedItemArray.Count();
	PyObject* resultList = PyList_New(count);

	// am using linked list here to return only a single python object per database handle
	CLinkedDatabaseList* linkedDatabases = NULL;
	for (int i = 0; i < count; i++)
	{
		TLmkItemIdDbCombiInfo selectedItem = selectedItemArray[i];
		CPosLandmarkDatabase* landmarkDatabase = selectedItem.GetLmDb();
		obj_LandmarkDatabase* database = NULL;
		if (linkedDatabases != NULL)
		{
			database = linkedDatabases->Find(landmarkDatabase);
		}
		if (database == NULL)
		{
			database = createDatabase(selectedItem.GetLmDb());
			CLinkedDatabaseList* newDatabase = new CLinkedDatabaseList(database);
			if (linkedDatabases == NULL)
			{
				linkedDatabases = newDatabase;
			}
			else
			{
				linkedDatabases->setNext(newDatabase);
			}
			Py_DECREF((PyObject*)database);
		}
		Py_INCREF((PyObject*)database);
		PyList_SetItem(resultList, i, Py_BuildValue("(iO)", selectedItem.GetItemId(), (PyObject*)database)); // Note: SetItem steals a reference
	}
	if (linkedDatabases) delete linkedDatabases;

	selectedItemArray.Close();
    return Py_BuildValue("(iO)", 1, resultList);
}

static PyObject* landmarks_ShowSelectCategoryDialog(PyObject* self, PyObject* args)
{
	return landmarks_ShowSelectDialog(self, args, 0);
}
static PyObject* landmarks_ShowSelectMultipleCategoryDialog(PyObject* self, PyObject* args)
{
	return landmarks_ShowSelectMultipleDialog(self, args, 0);
}
static PyObject* landmarks_ShowSelectLandmarkDialog(PyObject* self, PyObject* args)
{
	return landmarks_ShowSelectDialog(self, args, 1);
}
static PyObject* landmarks_ShowSelectMultipleLandmarkDialog(PyObject* self, PyObject* args)
{
	return landmarks_ShowSelectMultipleDialog(self, args, 1);
}

static PyObject* landmarks_ReleaseLandmarkResources(PyObject* /*self*/, PyObject* /*args*/)
{
	ReleaseLandmarkResources(NULL);
	RETURN_PYNONE
}

static PyObject* landmarks_PackLandmark(PyObject* /*self*/, PyObject* args)
{
#ifdef __SERIES60_31__
	obj_Landmark* landmark;
	if (PyArg_ParseTuple(args, "O!:PackLandmark", &type_Landmark, &landmark)) {
		PyObject* result;
	    TRAPD(err, 
	    	HBufC8* buf = PosLandmarkSerialization::PackL(*landmark->iLandmark);
	    	result = Py_BuildValue("s#", buf->Ptr(), buf->Length());
	    );
	
	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    return result;
	}
	return 0;
#else
    PyErr_SetString(PyExc_TypeError, "Serialization unavailable, need Symbian 3rd FP1 or higher");
    return 0;
#endif	
}

static PyObject* landmarks_UnpackLandmark(PyObject* /*self*/, PyObject* args)
{
#ifdef __SERIES60_31__
	char* data;
	int dataLen;
	if (PyArg_ParseTuple(args, "s#:UnpackLandmark", &data, &dataLen)) {
        TPtr8 dataDesc((TUint8*)data, dataLen);
	    obj_Landmark *landmark = PyObject_New(obj_Landmark, &type_Landmark);
	
	    if (!landmark)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }
	    TRAPD(err, 
	    	landmark->iLandmark = PosLandmarkSerialization::UnpackL(dataDesc);
	    );
	
	    if (err)
	    {
	        PyObject_Del(landmark);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

    	return (PyObject*)landmark;
	}
	return 0;
#else
    PyErr_SetString(PyExc_TypeError, "Serialization unavailable, need Symbian 3rd FP1 or higher");
    return 0;
#endif	
}

static PyObject* landmarks_PackCategory(PyObject* /*self*/, PyObject* args)
{
#ifdef __SERIES60_31__
	obj_LandmarkCategory* category;
	if (PyArg_ParseTuple(args, "O!:PackCategory", &type_LandmarkCategory, &category)) {
		PyObject* result;
	    TRAPD(err, 
	    	HBufC8* buf = PosLmCategorySerialization::PackL(*category->iLandmarkCategory);
	    	result = Py_BuildValue("s#", buf->Ptr(), buf->Length());
	    );
	
	    if (err)
	    {
	        return SPyErr_SetFromSymbianOSErr(err);
	    }
	    return result;
	}
	return 0;
#else
    PyErr_SetString(PyExc_TypeError, "Serialization unavailable, need Symbian 3rd FP1 or higher");
    return 0;
#endif	
}

static PyObject* landmarks_UnpackCategory(PyObject* /*self*/, PyObject* args)
{
#ifdef __SERIES60_31__
	char* data;
	int dataLen;
	if (PyArg_ParseTuple(args, "s#:UnpackCategory", &data, &dataLen)) {
        TPtr8 dataDesc((TUint8*)data, dataLen);
	    obj_LandmarkCategory *category = PyObject_New(obj_LandmarkCategory, &type_LandmarkCategory);
	
	    if (!category)
	    {
	        PyErr_SetString(PyExc_TypeError, "Alloc error");
	        return 0;
	    }
	    TRAPD(err, 
	    	category->iLandmarkCategory = PosLmCategorySerialization::UnpackL(dataDesc);
	    );
	
	    if (err)
	    {
	        PyObject_Del(category);
	        return SPyErr_SetFromSymbianOSErr(err);
	    }

    	return (PyObject*)category;
	}
	return 0;
#else
    PyErr_SetString(PyExc_TypeError, "Serialization unavailable, need Symbian 3rd FP1 or higher");
    return 0;
#endif	
}

/// DLL ///////////////////////////////////////////////////////////////////////////////////////////

static const PyMethodDef module_methods[] =
{
    {"OpenDefaultDatabase", (PyCFunction)landmarkdatabase_OpenDefaultLandmarkDatabase, METH_NOARGS},
    {"OpenDatabase", (PyCFunction)landmarkdatabase_OpenLandmarkDatabase, METH_VARARGS},
    {"CreateAreaCriteria", (PyCFunction)searchcriteria_CreateAreaCriteria, METH_VARARGS},
    {"CreateCatNameCriteria", (PyCFunction)searchcriteria_CreateCatNameCriteria, METH_VARARGS},
    {"CreateCategoryCriteria", (PyCFunction)searchcriteria_CreateCategoryCriteria, METH_VARARGS},
    {"CreateCompositeCriteria", (PyCFunction)searchcriteria_CreateCompositeCriteria, METH_VARARGS},
    {"CreateIdListCriteria", (PyCFunction)searchcriteria_CreateIdListCriteria, METH_VARARGS},
    {"CreateNearestCriteria", (PyCFunction)searchcriteria_CreateNearestCriteria, METH_VARARGS},
    {"CreateTextCriteria", (PyCFunction)searchcriteria_CreateTextCriteria, METH_VARARGS},
    {"CreateLandmark", (PyCFunction)landmark_CreateLandmark, METH_NOARGS},
    {"CreateLandmarkCategory", (PyCFunction)landmarkcategory_CreateLandmarkCategory, METH_NOARGS},
    {"ShowSelectCategoryDialog", (PyCFunction)landmarks_ShowSelectCategoryDialog, METH_VARARGS},
    {"ShowSelectMultipleCategoryDialog", (PyCFunction)landmarks_ShowSelectMultipleCategoryDialog, METH_VARARGS},
    {"ShowSelectLandmarkDialog", (PyCFunction)landmarks_ShowSelectLandmarkDialog, METH_VARARGS},
    {"ShowSelectMultipleLandmarkDialog", (PyCFunction)landmarks_ShowSelectMultipleLandmarkDialog, METH_VARARGS},
    {"ReleaseLandmarkResources", (PyCFunction)landmarks_ReleaseLandmarkResources, METH_NOARGS},
    {"CreateDatabaseManager", (PyCFunction)databasemanager_CreateDatabaseManager, METH_NOARGS},
    {"CreateDatabaseInfo", (PyCFunction)databaseinfo_CreateDatabaseInfo, METH_VARARGS},
    {"PackLandmark", (PyCFunction)landmarks_PackLandmark, METH_VARARGS},
    {"UnpackLandmark", (PyCFunction)landmarks_UnpackLandmark, METH_VARARGS},
    {"PackCategory", (PyCFunction)landmarks_PackCategory, METH_VARARGS},
    {"UnpackCategory", (PyCFunction)landmarks_UnpackCategory, METH_VARARGS},
    {"CreateLandmarkEncoder", (PyCFunction)landmarkencoder_CreateLandmarkEncoder, METH_VARARGS},
    {"CreateLandmarkParser", (PyCFunction)landmarkparser_CreateLandmarkParser, METH_VARARGS},
	{0, 0} /* sentinel */
};

static bool createType(PyTypeObject typeTemplate, char* type_string)
{
	// do type
	PyTypeObject *newTypeObject = PyObject_New(PyTypeObject, &PyType_Type);
	if (!newTypeObject)
	{
   		PyErr_SetString(PyExc_Exception, "Alloc error");
		return false;
	}

    *newTypeObject = typeTemplate;

	TInt err = SPyAddGlobalString(type_string, (PyObject*)newTypeObject);
    if (0 != err) // 0 is success
	{
        PyObject_Del(newTypeObject);
   		PyErr_SetString(PyExc_Exception, "SPyAddGlobalString failed");
        return false;
	}

    // set the type of the type object to .... type!
    newTypeObject->ob_type = &PyType_Type;

    return true;
}

DL_EXPORT(void) init_landmarks()
{
  	// 3rd edition, use _landmarks for separate python loader
	Py_InitModule("_landmarks", (PyMethodDef*) module_methods);

	// create/register types
	if (!createType(type_template_LandmarkDatabase, landmarkdatabase_type_string)) return;
	if (!createType(type_template_SearchCriteria, searchcriteria_type_string)) return;
	if (!createType(type_template_LandmarkSearch, landmarksearch_type_string)) return;
	if (!createType(type_template_LmOperation, lmoperation_type_string)) return;
	if (!createType(type_template_ItemIterator, itemiterator_type_string)) return;
	if (!createType(type_template_Landmark, landmark_type_string)) return;
	if (!createType(type_template_LandmarkCategory, landmarkcategory_type_string)) return;
	if (!createType(type_template_CategoryManager, categorymanager_type_string)) return;
	if (!createType(type_template_DatabaseManager, databasemanager_type_string)) return;
	if (!createType(type_template_DatabaseInfo, databaseinfo_type_string)) return;
	if (!createType(type_template_DatabaseSettings, databasesettings_type_string)) return;
	if (!createType(type_template_LandmarkEncoder, landmarkencoder_type_string)) return;
	if (!createType(type_template_LandmarkParser, landmarkparser_type_string)) return;
	if (!createType(type_template_CBufBase, cbufbase_type_string)) return;
}
